using AweCoreDemo.Data;
using AweCoreDemo.Models;
using AweCoreDemo.Utils.Awesome;
using AweCoreDemo.ViewModels.Display;
using Microsoft.AspNetCore.Mvc;

using Omu.Awem.Utils;
using Omu.AwesomeMvc;

#pragma warning disable CS1998 // Async method lacks 'await' operators and will run synchronously
namespace AweCoreDemo.Controllers.Demos.Grid
{
    public class GridFilterRowServerSideDataController : Controller
    {
        public IActionResult Index()
        {
            return View();
        }

        public IActionResult LargeDataSource()
        {
            return View();
        }

        private static object MapToGridModel(Lunch o)
        {
            return new
            {
                o.Id,
                o.Person,
                DateStr = o.Date.ToShortDateString(),
                CountryName = o.Country.Name,
                ChefName = o.Chef.FullName,
                MealsStr = string.Join(", ", o.Meals.Select(m => m.Name)),
                FoodName = o.Food.Name,
                FoodPic = o.Food.Pic,
                Price = o.Price,
                Organic = o.Organic ? "Yes" : "No"
            };
        }

        public class LunchFilter : FilterPrm
        {
            public DateTime? Date { get; set; }
            public string DateOp { get; set; }

            public string Person { get; set; }

            public decimal? Price { get; set; }
            public string PriceOp { get; set; }

            public int? Food { get; set; }
            public int? Country { get; set; }
            public int? Chef { get; set; }
            public int[] Meals { get; set; }
            public bool? Organic { get; set; }
        }

        public IActionResult LunchFilterGrid(GridParams g, LunchFilter filter)
        {
            var query = Db.Lunches.AsQueryable();

            var filterBuilder = new FilterBuilder<Lunch>
            {
                Filter = filter,
                Query = query
            };

            filterBuilder
                .StringFilter(nameof(LunchFilter.Person))
                .OpFilter<Lunch, decimal>(nameof(LunchFilter.Price))
                .OpFilter<Lunch, DateTime>(nameof(LunchFilter.Date))
                .Add(nameof(LunchFilter.Country), new FilterRule<Lunch>
                {
                    Query = o => o.Country.Id == filter.Country,
                    GetData = async q =>
                    {
                        return (q.Select(o => o.Country).Distinct().ToArray())
                        .Select(o => new KeyContent(o.Id, o.Name));
                    }
                })
                .Add(nameof(LunchFilter.Chef), new FilterRule<Lunch>
                {
                    Query = o => o.Chef.Id == filter.Chef,
                    GetData = async q =>
                    {
                        return (q.Select(o => o.Chef).Distinct().ToArray())
                            .Select(o => new KeyContent(o.Id, o.FullName));
                    }
                })
                .Add(nameof(LunchFilter.Food), new FilterRule<Lunch>
                {
                    Query = o => o.Food.Id == filter.Food,
                    // no need for data,  Url set in the view for the Food DropdownListOpt
                })
                .Add(nameof(LunchFilter.Meals), new FilterRule<Lunch>
                {
                    QueryFunc = () =>
                    {
                        var ids = filter.Meals;
                        var count = ids.Count();
                        return o => o.Meals.Count(m => ids.Contains(m.Id)) == count;
                    },

                    GetData = async q =>
                    {
                        return (q.SelectMany(o => o.Meals).Distinct().ToArray())
                            .OrderBy(o => o.Id)
                            .Select(o => new KeyContent(o.Id, o.Name));
                    },

                    // get data after querying this time,
                    // to filter the meals in GetData by previously selected meals
                    SelfQuery = true
                });

            query = filterBuilder.ApplyAsync().Result;

            var gmb = new GridModelBuilder<Lunch>(query, g)
            {
                KeyProp = o => o.Id,
                Map = MapToGridModel,
                Tag = filterBuilder.GetGridData()
            };

            return Json(gmb.Build());
        }

        public class DinnerFilterPrm : FilterPrm
        {
            public int? Date { get; set; }
            public string Name { get; set; }
            public int? Chef { get; set; }
            public int[] Meals { get; set; }
            public bool? Organic { get; set; }
        }

        #region Outside filter row

        public IActionResult DinnersFilterGrid(GridParams g, DinnerFilterPrm filter)
        {
            var query = Db.Dinners.AsQueryable();

            var filterBuilder = new FilterBuilder<Dinner>();
            filterBuilder.Query = query;
            filterBuilder.Filter = filter;


            filterBuilder
                .StringFilter("Name")
                .Add("Date", new FilterRule<Dinner>
                {
                    Query = o => o.Date.Year == filter.Date,

                    GetData = async q =>
                    {
                        var list = new List<KeyContent>
                        {
                            new KeyContent("", "all years")
                        };

                        var years = (q.Select(o => o.Date.Year).Distinct().ToArray()).OrderBy(o => o);

                        list.AddRange(AweUtil.ToKeyContent(years));

                        return list.ToArray();
                    }
                })
                .Add("Chef", new FilterRule<Dinner>()
                {
                    Query = o => o.Chef.Id == filter.Chef,
                    GetData = async q =>
                    {
                        return (q.Select(o => o.Chef).Distinct().ToArray())
                            .Select(o => new KeyContent(o.Id, o.FullName));
                    }
                })
                .Add("Meals", new FilterRule<Dinner>
                {
                    QueryFunc = () =>
                    {
                        var ids = filter.Meals;
                        var count = ids.Count();
                        return itm => itm.Meals.Count(m => ids.Contains(m.Id)) == count;
                    },

                    GetData = async q =>
                    {
                        return (q.SelectMany(o => o.Meals).Distinct().ToArray())
                            .OrderBy(o => o.Id)
                            .Select(o => new KeyContent(o.Id, o.Name));
                    },

                    // get data after querying this time, to filter the meals items as well
                    SelfQuery = true
                })
                .Add("Organic", new FilterRule<Dinner>
                {
                    Query = o => o.Organic == filter.Organic,

                    GetData = async q =>
                    {
                        var list = new List<KeyContent> { new KeyContent("", "all") };
                        list.AddRange(q.Select(o => o.Organic).Distinct()
                            .Select(o => new KeyContent(o, o ? "Yes" : "No")));

                        return list.ToArray();
                    }
                });

            // apply rules present in forder (touched by the user)
            query = filterBuilder.ApplyAsync().Result;

            var gmb = new GridModelBuilder<Dinner>(query, g)
            {
                KeyProp = o => o.Id,
                Map = DinnerMapToGridModel,
                Tag = filterBuilder.GetGridData()
            };

            return Json(gmb.Build());
        }

        private static object DinnerMapToGridModel(Dinner o)
        {
            return new
            {
                o.Id,
                o.Name,
                Date = o.Date.ToShortDateString(),
                ChefName = o.Chef.FullName,
                Meals = string.Join(", ", o.Meals.Select(m => m.Name)),
                Organic = o.Organic ? "Yes" : "No"
            };
        }
        #endregion

        #region Large data source
        /*beginlrgd*/
        public IActionResult LunchFilterGridLarge(GridParams g, LunchFilter filter)
        {
            var query = Db.Lunches.AsQueryable();

            var filterBuilder = new FilterBuilder<Lunch>();

            setupLunchFilter(filterBuilder, query, filter);

            query = filterBuilder.ApplyAsync().Result;

            var gmb = new GridModelBuilder<Lunch>(query, g)
            {
                KeyProp = o => o.Id,
                Map = MapToGridModel,
                Tag = filterBuilder.GetGridData()
            };

            return Json(gmb.Build());
        }

        private void setupLunchFilter(
            FilterBuilder<Lunch> filterBuilder,
            IQueryable<Lunch> query,
            LunchFilter filter)
        {
            filterBuilder.Query = query;
            filterBuilder.Filter = filter;

            filterBuilder
                .StringFilter(nameof(LunchFilter.Person))

                // Food filter dropdown is not using the grid filter query to get data
                // so it is possible to select an item from this filter dropdown
                // that will give us zero results in the grid (if there are other filters set)
                .Add(nameof(LunchFilter.Food), new FilterRule<Lunch> { Query = itm => itm.Food.Id == filter.Food })

                // returning data for the selected filter values
                .Add(nameof(LunchFilter.Country),
                new FilterRule<Lunch>
                {
                    Query = itm => itm.Country.Id == filter.Country,

                    GetData = async q =>
                    {
                        if (filter.Country == null) return null;

                        return (q.Select(o => o.Country).Where(o => o.Id == filter.Country).Distinct().ToArray())
                            .Select(o => new KeyContent(o.Id, o.Name));
                    }
                })

                .Add(nameof(LunchFilter.Meals),
                new FilterRule<Lunch>
                {
                    QueryFunc = () =>
                    {
                        var ids = filter.Meals;
                        var count = ids.Count();
                        return itm => itm.Meals.Count(m => ids.Contains(m.Id)) == count;
                    },

                    GetData = async q =>
                    {
                        if (filter.Meals == null) return null;

                        return (q.SelectMany(o => o.Meals).Where(o => filter.Meals.Contains(o.Id)).Distinct().ToArray())
                            .OrderBy(o => o.Id)
                            .Select(o => new KeyContent(o.Id, o.Name));
                    },

                    // get data after "Meals" QueryFunc has been applied to the Query (q)
                    SelfQuery = true
                })

                .Add(nameof(LunchFilter.Chef),
                new FilterRule<Lunch>
                {
                    Query = itm => itm.Chef.Id == filter.Chef,

                    GetData = async q =>
                    {
                        if (filter.Chef == null) return null;

                        return (q.Select(lunch => lunch.Chef)
                            .Where(chef => chef.Id == filter.Chef)
                            .Take(1)
                            .ToArray())
                            .Select(o => new KeyContent(o.Id, o.FullName));
                    }
                });
        }

        public IActionResult SearchFood(string term)
        {
            var items = (Db.Foods
                .AsQueryable()
                .Filter(term, o => o.Name)
                .Take(10)
                .ToArray())
                .Select(o => new MealDisplay(o.Id, o.Name, o.Pic));

            return Json(items);
        }

        public IActionResult GetFoodsInit(int? v)
        {
            var query = Db.Foods.OrderBy(o => o.Id);
            var items = query.Take(3).ToList();
            var selected = query.FirstOrDefault(o => o.Id == v);

            if (selected != null && !items.Contains(selected))
            {
                items.Add(selected);
            }

            var lst = new List<MealDisplay> { new MealDisplay("", "any", "pasta.png") };
            lst.AddRange(items.Select(o => new MealDisplay(o.Id, o.Name, o.Pic)));

            return Json(lst);
        }

        public IActionResult SearchCountry(LunchFilter filter, string term)
        {
            var query = Db.Lunches.AsQueryable();
            var fb = new FilterBuilder<Lunch>();

            setupLunchFilter(fb, query, filter);

            query = fb.GetQueryFor(nameof(LunchFilter.Country));

            var items = (query.Select(o => o.Country)
                .Filter(term, o => o.Name)
                .Distinct()
                .Take(10)
                .ToArray())
                .Select(o => new KeyContent(o.Id, o.Name));

            return Json(items);
        }

        public IActionResult SearchMeals(string term, LunchFilter filter)
        {
            var query = Db.Lunches.AsQueryable();

            var fb = new FilterBuilder<Lunch>();

            // apply other grid filters in order
            setupLunchFilter(fb, query, filter);

            query = fb.GetQueryFor(nameof(LunchFilter.Meals));

            var data = (query.SelectMany(o => o.Meals)
                .Filter(term, o => o.Name)
                .Distinct()
                .Take(10)
                .OrderBy(o => o.Id)
                .ToArray())
                .Select(o => new KeyContent(o.Id, o.Name));

            return Json(data);
        }

        public IActionResult SearchChef(string term, LunchFilter fp)
        {
            var query = Db.Lunches.AsQueryable();

            var fb = new FilterBuilder<Lunch>();

            setupLunchFilter(fb, query, fp);

            query = fb.GetQueryFor(nameof(LunchFilter.Chef));

            var data = (query.Select(o => o.Chef)
                            .Filter(term, o => o.FirstName, o => o.LastName)
                            .Distinct()
                            .Take(10)
                            .OrderBy(o => o.Id)
                            .ToArray())
                            .Select(o => new KeyContent(o.Id, o.FullName));

            return Json(data);
        }
        #endregion
    }
}

#pragma warning restore CS1998 // Async method lacks 'await' operators and will run synchronously