using AweCoreDemo.Data;
using AweCoreDemo.Models;
using AweCoreDemo.ViewModels.Input;
using Microsoft.AspNetCore.Mvc;

using Omu.AwesomeMvc;

#pragma warning disable CS1998 // Async method lacks 'await' operators and will run synchronously
namespace AweCoreDemo.Controllers.Demos.Grid
{
    public class TreeGridController : Controller
    {
        public IActionResult Index()
        {
            return View();
        }

        #region Simple tree
        public IActionResult SimpleTree(GridParams g)
        {
            var rootNodes = Db.TreeNodes.Where(o => o.Parent == null).AsQueryable();

            var builder = new GridModelBuilder<TreeNode>(rootNodes, g)
            {
                KeyProp = o => o.Id,
                GetChildrenAsync = async (node, nodeLevel) =>
                    Db.TreeNodes.Where(n => n.Parent == node).ToArray(),
                Map = o => new { o.Name, o.Id }
            };

            return Json(builder.Build());
        }
        #endregion

        #region Lazy loading tree

        public IActionResult LazyTree(GridParams g)
        {
            var rootNodes = Db.TreeNodes.Where(o => o.Parent == null).AsQueryable();

            var builder = new GridModelBuilder<TreeNode>(rootNodes, g)
            {
                KeyProp = o => o.Id,
                GetChildrenAsync = async (node, nodeLevel) =>
                    {
                        var children = Db.TreeNodes.Where(o => o.Parent == node).ToArray();

                        // set this node as lazy when it's above level 1 (relative), it has children, and this is not a lazy request already
                        if (nodeLevel > 1 && children.Any() && g.Key == null) return Awe.LazyNode;

                        return children;
                    },
                GetItem = () => // used for lazy loading
                    {
                        var id = Convert.ToInt32(g.Key);
                        return Db.TreeNodes.First(o => o.Id == id);
                    },
                Map = MapNode
            };

            return Json(builder.Build());
        }

        #endregion

        #region Crud Tree
        /*begin3*/
        public IActionResult CrudTree(GridParams g, string name = "")
        {
            var all = Db.TreeNodes.ToList();

            var sresult = all.Where(o => o.Name.Contains(name)).ToList();

            var result = sresult.ToList();

            foreach (var treeNode in sresult)
            {
                AddParentsTo(result, treeNode);
            }

            var roots = result.Where(o => o.Parent == null).AsQueryable();

            var gmb = new GridModelBuilder<TreeNode>(roots, g)
            {
                KeyProp = o => o.Id,
                Map = MapNode
            };

            gmb.GetItemAsync = async () =>
            {
                var key = Convert.ToInt32(gmb.GridParams.Key);
                return Db.Find<TreeNode>(key);
            };

            gmb.GetChildren = (node, nodeLevel) =>
            {
                var children = result.Where(o => o.Parent == node);
                return gmb.OrderBy(children.AsQueryable());
            };

            return Json(gmb.Build());
        }

        private object MapNode(TreeNode node)
        {
            return new { node.Name, node.Id };
        }

        private void AddParentsTo(ICollection<TreeNode> result, TreeNode node)
        {
            if (node.Parent != null)
            {
                if (!result.Contains(node.Parent))
                {
                    result.Add(node.Parent);
                    AddParentsTo(result, node.Parent);
                }
            }
        }

        public IActionResult Create(int? parentId)
        {
            return PartialView(new TreeNodeInput { ParentId = parentId });
        }

        [HttpPost]
        public IActionResult Create(TreeNodeInput input)
        {
            if (!ModelState.IsValid) return PartialView(input);

            var parent = input.ParentId.HasValue ? Db.Find<TreeNode>(input.ParentId) : null;
            var node = new TreeNode { Name = input.Name, Parent = parent };

            Db.Add(node);



            var result = new
            {
                node.Id // used to flash the new row if visibile
            };

            return Json(result);
        }

        public IActionResult Edit(int id)
        {
            var node = Db.Find<TreeNode>(id);
            return PartialView("Create", new TreeNodeInput { Id = node.Id, Name = node.Name });
        }

        [HttpPost]
        public IActionResult Edit(TreeNodeInput input)
        {
            if (!ModelState.IsValid) return PartialView("Create", input);

            var node = Db.Find<TreeNode>(input.Id);
            node.Name = input.Name;

            return Json(new { node.Id });
        }

        public IActionResult Delete(int id)
        {
            var node = Db.Find<TreeNode>(id);

            return PartialView(new DeleteConfirmInput
            {
                Id = id,
                Type = "node",
                Name = node.Name
            });
        }

        [HttpPost]
        public IActionResult Delete(DeleteConfirmInput input)
        {
            var node = Db.Find<TreeNode>(input.Id);
            DeleteNodeTree(node);

            var result = new
            {
                node.Id
            };

            return Json(result);
        }

        private void DeleteNodeTree(TreeNode node)
        {
            var children = Db.TreeNodes.Where(o => o.Parent == node).ToList();

            foreach (var child in children)
            {
                DeleteNodeTree(child);
            }

            Db.Remove(Db.Find<TreeNode>(node.Id));
        }
        /*end3*/
        #endregion
    }
}
#pragma warning restore CS1998 // Async method lacks 'await' operators and will run synchronously