using AweCoreDemo.Utils;
using Microsoft.AspNetCore.Html;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Rendering;
using Microsoft.AspNetCore.Mvc.Routing;
using Microsoft.AspNetCore.Mvc.ViewFeatures;

namespace AweCoreDemo.Helpers
{
    public static class VldHelpers
    {
        public static IDictionary<string, IDictionary<string, string>> GetVldAttrs<TModel>(this IHtmlHelper html)
        {
            var res = new Dictionary<string, IDictionary<string, string>>();

            var attributeProvider = html.ViewContext.HttpContext.RequestServices.GetRequiredService<ValidationHtmlAttributeProvider>();
            var modelExplorer = html.MetadataProvider.GetModelExplorerForType(typeof(TModel), null);

            foreach (var prop in typeof(TModel).GetProperties())
            {
                var clientAttributes = new Dictionary<string, string>();

                attributeProvider.AddValidationAttributes(html.ViewContext, modelExplorer.GetExplorerForProperty(prop.Name), clientAttributes);

                res.Add(prop.Name, clientAttributes);
            }

            return res;
        }

        public static IHtmlContent GetVldRules<TModel>(this IHtmlHelper html, string sel)
        {
            var result = string.Empty;
            var attributeProvider = html.ViewContext.HttpContext.RequestServices.GetRequiredService<ValidationHtmlAttributeProvider>();

            foreach (var prop in typeof(TModel).GetProperties())
            {
                var attributes = new Dictionary<string, string>();

                var modelExplorer = html.MetadataProvider.GetModelExplorerForType(typeof(TModel), null);
                attributeProvider.AddValidationAttributes(html.ViewContext, modelExplorer.GetExplorerForProperty(prop.Name), attributes);

                var rules = new Dictionary<string, string>();

                var kreq = "data-val-required";
                var klen = "data-val-length";
                var klenmax = "data-val-length-max";
                var klenmin = "data-val-length-min";

                if (attributes.ContainsKey(kreq))
                {
                    var msg = attributes[kreq];
                    rules.Add("ovld.rules.req", msg.ToString());
                }

                if (attributes.ContainsKey(klen))
                {
                    var msg = attributes[klen];
                    var max = attributes.ContainsKey(klenmax) ? attributes[klenmax] : string.Empty;
                    var min = attributes.ContainsKey(klenmin) ? attributes[klenmin] : string.Empty;
                    rules.Add($"ovld.rules.len({max}, {min})", msg.ToString());
                }

                if (!rules.Any()) continue;

                result += $"'{prop.Name}': [";

                result = rules.Select(kv => "chk:" + kv.Key + ", msg:'" + JsEncode(kv.Value) + "'").Aggregate(result, (current, s) => current + "{" + s + "},");

                result += "],";
            }

            return new HtmlString($"<script>ovld.bind({{ " +
                                     $"subev: 'aweinlinebfsave', " +
                                     $"sel: '{sel}', rules:{{{result}}}, " +
                                     $"gmsgc: function(inp){{ return inp.closest('.oinlc').find('.o-gvalidmsg'); }} " +
                                     $"}});</script>");
        }

        internal static string JsEncode(string s)
        {
            if (s == null) return null;
            return s.Replace("'", "\\'").Replace("\"", "&quot;");
        }
    }

    public static class ShowCodeHelpers
    {
        /// <summary>
        /// get the string value of code between the /*begin*/ and /*end*/ comment blocks  
        /// </summary>
        /// <param name="html"></param>
        /// <param name="path"> file location path</param>
        /// <param name="k"></param>
        /// <returns></returns>
        public static IHtmlContent Csrc(this IHtmlHelper html, string path, object k = null)
        {
            var startWord = "/*begin" + k + "*/";
            var endWord = "/*end" + k + "*/";

            var newpath = Path.Combine(html.ServerMapPath(), path);
            var lines = ReadAllLines(newpath);
            var code = StrUtil.GetCode(lines, startWord, endWord);

            return new HtmlString(StrUtil.ParseStrToCode(code, path));
        }

        public static CodeHelper Util(this IHtmlHelper html, string path)
        {
            return new CodeHelper(html, path).Util();
        }

        public static CodeHelper Code(this IHtmlHelper html, string path)
        {
            return new CodeHelper(html, path);
        }

        /// <summary>
        /// get the string value of the controller code between the /*begin(key)*/ and /*end(key)*/ comment blocks 
        /// </summary>
        /// <param name="html"></param>
        /// <param name="path"></param>
        /// <param name="k">key</param>
        /// <returns></returns>
        public static IHtmlContent Csource(this IHtmlHelper html, string path, object k = null)
        {
            var startWord = "/*begin" + k + "*/";
            var endWord = "/*end" + k + "*/";

            var newpath = Path.Combine(html.ServerMapPath(), "Controllers", path);
            var lines = ReadAllLines(newpath);

            var code = StrUtil.GetCode(lines, startWord, endWord);

            return new HtmlString(StrUtil.ParseStrToCode(code, path));
        }

        /// <summary>
        /// get the string value of js file between the /*begin*/ and /*end*/ comment blocks 
        /// </summary>
        /// <param name="html"></param>
        /// <param name="path"></param>
        /// <param name="k">key</param>
        /// <returns></returns>
        public static IHtmlContent SourceJs(this IHtmlHelper html, string path, object k = null)
        {
            var startWord = "/*begin" + k + "*/";
            var endWord = "/*end" + k + "*/";

            var newpath = Path.Combine(html.ServerMapPath(), "wwwroot", "js", path);
            var lines = ReadAllLines(newpath);
            var code = StrUtil.GetCode(lines, startWord, endWord);

            return new HtmlString(StrUtil.ParseStrToCode(code, path));
        }

        /// <summary>
        /// get the string value of the view code located between the begin+key and end+key comment blocks
        /// </summary>
        /// <param name="html"></param>
        /// <param name="path"></param>
        /// <param name="k">key</param>
        /// <param name="wrap">wrap with div class='code'</param>
        /// <returns></returns>
        public static IHtmlContent Source(this IHtmlHelper html, string path, object k = null, bool wrap = false)
        {
            var key = k == null ? "" : k.ToString();
            var newpath = Path.Combine(html.ServerMapPath(), "Views", path);
            var lines = ReadAllLines(newpath);


            var code = path.EndsWith(".cshtml") ? StrUtil.GetCode(lines, "@*begin" + key + "*@", "@*end" + key + "*@") : StrUtil.GetCode(lines, "<%--begin" + key + "--%>", "<%--end" + key + "--%>");
            var result = StrUtil.ParseStrToCode(code, path);
            if (wrap) result = "<div class='code'>" + result + "</div>";

            return new HtmlString(result);
        }

        private static string[] ReadAllLines(string path)
        {
            try
            {
                var lines = File.ReadAllLines(path);
                return lines;
            }
            catch (Exception ex)
            {
                return new[] { ex.Message };
            }
        }
    }

}