﻿using AweMySql.Models;
using AweMySql.Utils.Awe;
using AweMySql.ViewModels.Input;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Omu.Awem.Utils;
using Omu.AwesomeMvc;

namespace AweMySql.Controllers
{
    public class ChefInlineEditController : Controller
    {
        private readonly MyDbContext myCx;

        public ChefInlineEditController(MyDbContext dbContext)
        {
            myCx = dbContext;
        }

        private object MapToRowModel(Chef o)
        {
            return new
            {
                o.Id,
                o.FirstName,
                o.LastName,
                CountryName = o.Country.Name,

                // below properties used for inline editing only
                o.CountryId
            };
        }

        public class ChefPrm : FilterPrm
        {
            public string FirstName { get; set; }
            public string LastName { get; set; }
            public int? Country { get; set; }
        }

        public async Task<IActionResult> Grid(GridParams g, ChefPrm filter)
        {
            var query = myCx.Chefs
                .Include(o => o.Country)
                .AsQueryable();

            var fb = new FilterBuilder<Chef>()
            {
                Query = query,
                Filter = filter
            };

            fb.StringFilter(nameof(ChefPrm.FirstName))
              .StringFilter(nameof(ChefPrm.LastName))
              .Add(nameof(ChefPrm.Country), new FilterRule<Chef>
                {
                    Query = o => o.Country.Id == filter.Country,
                    GetData = async q =>
                    {
                        return (await q.Select(o => o.Country).Distinct().ToArrayAsync())
                        .Select(o => new KeyContent(o.Id, o.Name));
                    }
                });

            query = await fb.ApplyAsync();

            var gmb = new GridModelBuilder<Chef>(query, g)
            {
                KeyProp = o => o.Id,
                Map = MapToRowModel,
                Tag = fb.GetGridData()
            };

            return Json(await gmb.EFBuildAsync());
        }

        [HttpPost]
        public async Task<ActionResult> Save(ChefInput input)
        {
            if (ModelState.IsValid)
            {
                var edit = input.Id.HasValue;
                var ent = edit ? await myCx.FindAsync<Chef>(input.Id) : new Chef();

                ent.FirstName = input.FirstName;
                ent.LastName = input.LastName;
                ent.CountryId = input.CountryId;

                if (!edit)
                {
                    myCx.Add(ent);
                }

                await myCx.SaveChangesAsync();

                await LoadChefAsync(ent);

                return Json(new { Item = MapToRowModel(ent) });
            }

            return Json(ModelState.GetErrorsInline());
        }

        public async Task<ActionResult> Delete(int id)
        {
            var chef = await myCx.FindAsync<Chef>(id);

            return PartialView(new DeleteConfirmInput
            {
                Id = id,
                Type = "chef",
                Name = chef.FirstName + " " + chef.LastName
            });
        }

        [HttpPost]
        public async Task<ActionResult> Delete(DeleteConfirmInput input)
        {
            await myCx.DeleteAsync<Chef>(input.Id);

            return Json(new { input.Id });
        }

        private async Task LoadChefAsync(Chef item)
        {
            await myCx.Chefs.Where(o => o == item).Include(o => o.Country).LoadAsync();
        }

        #region for aweui page only
        public async Task<ActionResult> GetChefsPage(GridParams g, string search)
        {
            //// g.Key is set when calling grid api.update(id) (on row Save)
            //if (g.Key != null)
            //{
            //    var item = await myCx.FindAsync<Chef>(Convert.ToInt32(g.Key));
            //    await LoadChefAsync(item);

            //    return Json(new { count = 1, items = new[] { MapToRowModel(item) } });
            //}

            search = (search ?? "").ToLower();
            var query = myCx.Chefs
                .Where(o => o.FirstName.Contains(search) || o.LastName.Contains(search))
                .Include(o => o.Country)
                .AsQueryable();

            // unmapping back the column.bind values to avoid gmb throwing that Chef has no CountryName prop
            var dict = new Dictionary<string, string> { { "CountryName", "Country.Name" } };
            unmap(g, dict);

            var gmb = new GridModelBuilder<Chef>(g)
            {
                KeyProp = o => o.Id
            };

            // ordering the data according to the grid params (g)
            query = gmb.OrderBy(query);

            var count = await query.CountAsync();
            var items = await gmb.GetPage(query).ToArrayAsync();
            var mappedItems = items.Select(MapToRowModel); // mapping to send only the needed data

            return Json(new { count, items = mappedItems });
        }
        private void unmap(GridParams g, IDictionary<string, string> dict)
        {
            if (g.SortNames != null)
            {
                g.SortNames = g.SortNames.Select(sn => dict.ContainsKey(sn) ? dict[sn] : sn).ToArray();
            }
        }
        #endregion
    }
}