﻿using System;
using System.Linq;
using AweRazorPages.Models;
using AweRazorPages.ViewModels.Input;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using Omu.AwesomeMvc;

namespace AweRazorPages.Pages
{
    public class CrudPopupModel : PageModel
    {
        private static object MapToGridModel(Dinner o)
        {
            return
                new
                {
                    o.Id,
                    o.Name,
                    Date = o.Date.ToShortDateString(),
                    ChefName = o.Chef.FirstName + " " + o.Chef.LastName,
                    Meals = string.Join(", ", o.Meals.Select(m => m.Name)),
                    BonusMealName = o.BonusMeal.Name
                };
        }

        public ActionResult OnPostGridGetItems(GridParams g, string search)
        {
            search = (search ?? "").ToLower();
            var items = Db.Dinners.Where(o => o.Name.ToLower().Contains(search)).AsQueryable();

            return new JsonResult(new GridModelBuilder<Dinner>(items, g)
            {
                KeyProp = o => o.Id, // needed for api select, update, tree, nesting, EF
                GetItem = () => Db.Get<Dinner>(Convert.ToInt32(g.Key)), // called by the grid.api.update ( edit popupform success js func )
                Map = MapToGridModel
            }.Build());
        }

        public ActionResult OnGetCreate()
        {
            // make sure to use "return PartialView" for PopupForm/Popup views 
            // this will ignore _viewstart.cshtml so that you don't use the _Layout.cshtml and reload all the scripts
            return Partial("Create");
        }

        public ActionResult OnPostCreate(DinnerInput input)
        {
            if (!ModelState.IsValid) return Partial("Create", input);

            var dinner = Db.Insert(new Dinner
            {
                Name = input.Name,
                Date = input.Date.Value,
                Chef = Db.Get<Chef>(input.ChefId),
                Meals = input.MealsIds.Select(mid => Db.Get<Meal>(mid)),
                BonusMeal = Db.Get<Meal>(input.BonusMealId)
            });

            // the create PopupForm's success function aweUtils.itemCreated expects the grid row model obj, to render and append the new row
            return new JsonResult(MapToGridModel(dinner));
        }

        public ActionResult OnGetEdit(int id)
        {
            var dinner = Db.Get<Dinner>(id);

            var input = new DinnerInput
            {
                Id = dinner.Id,
                Name = dinner.Name,
                ChefId = dinner.Chef.Id,
                Date = dinner.Date,
                MealsIds = dinner.Meals.Select(o => o.Id),
                BonusMealId = dinner.BonusMeal.Id
            };

            return Partial("Create", input);
        }

        public ActionResult OnPostEdit(DinnerInput input)
        {
            if (!ModelState.IsValid) return Partial("Create", input);
            var dinner = Db.Get<Dinner>(input.Id);

            dinner.Name = input.Name;
            dinner.Date = input.Date.Value;
            dinner.Chef = Db.Get<Chef>(input.ChefId);
            dinner.Meals = input.MealsIds.Select(mid => Db.Get<Meal>(mid));
            dinner.BonusMeal = Db.Get<Meal>(input.BonusMealId);
            Db.Update(dinner);

            // the edit PopupForm's success function aweUtils.itemEdited expects an obj with "Id" property
            return new JsonResult(new { Id = dinner.Id });
        }

        public IActionResult OnGetDelete(int id)
        {
            var dinner = Db.Get<Dinner>(id);

            return Partial("Delete", new DeleteConfirmInput
            {
                Id = id,
                Type = "dinner",
                Name = dinner.Name
            });
        }

        public IActionResult OnPostDelete(DeleteConfirmInput input)
        {
            Db.Delete<Dinner>(input.Id);

            // the PopupForm's success function aweUtils.itemDeleted expects an obj with "Id" property
            return new JsonResult(new { input.Id });
        }
    }
}