﻿using Microsoft.AspNetCore.Mvc;
using MinSetupCore.Models;
using MinSetupCore.ViewModels.Input;
using Omu.AwesomeMvc;
using MinSetupCore.Data;
using Microsoft.EntityFrameworkCore;
using MinSetupCore.Utils.Awesome;
using Omu.Awem.Utils;

namespace MinSetupCore.Controllers
{
    public class GridInlineEditDemoController : Controller
    {
        private readonly MyContext mcx;

        public GridInlineEditDemoController(MyContext mcx)
        {
            this.mcx = mcx;
        }

        public IActionResult Index()
        {
            return View();
        }

        private object MapToGridModel(Dinner o)
        {
            return new
            {
                o.Id,
                o.Name,
                Date = o.Date.ToShortDateString(),
                ChefName = o.Chef.FullName,
                Meals = string.Join(", ", o.Meals.Select(m => m.Name)),
                BonusMeal = o.BonusMeal.Name,
                o.Organic,
                DispOrganic = o.Organic ? "Yes" : "No",

                // below properties used for inline editing only
                MealsIds = o.Meals.Select(m => m.Id).ToArray(),
                ChefId = o.Chef.Id,
                BonusMealId = o.BonusMeal.Id,

                // for conditional demo
                Editable = o.Meals.Count() > 1,
                DateReadOnly = o.Date.Year < 2015
            };
        }

        public async Task<IActionResult> GridGetItems(GridParams g, string search = "")
        {
            var query = mcx.Dinners
                .Include(o => o.Chef)
                .Include(o => o.Meals)
                .Include(o => o.Country)
                .Include(o => o.BonusMeal)
            .Where(o => o.Name.Contains(search)).AsQueryable();

            var model = new GridModelBuilder<Dinner>(query, g)
            {
                KeyProp = o => o.Id, // needed for api select, update, tree, nesting, EF                
                Map = MapToGridModel,
            };

            return Json(await model.EFBuildAsync());
            // you can use .Build() for non EF datasource (like a List<Dinner> query)
            // EFBuildAsync - custom extension for EF, you can copy it from demo source code
        }

        [HttpPost]
        public async Task<IActionResult> Save(DinnerInput input)
        {
            // custom validation example
            if (ModelState.IsValid && input.Name.Contains("asdf"))
            {
                ModelState.AddModelError("Name", "Name can't contain asdf");
            }

            if (ModelState.IsValid)
            {
                var isCreate = !input.Id.HasValue;

                var ent = isCreate ? new Dinner() :
                    await mcx.Dinners
                        .Include(o => o.Meals)
                        .FirstOrDefaultAsync(o => o.Id == input.Id);

                if (ent == null)
                {
                    throw new Exception("Item doesn't exist anymore, id:" + input.Id);
                }

                ent.Name = input.Name;
                ent.Date = input.Date.Value;
                ent.Chef = await mcx.FindAsync<Chef>(input.ChefId);

                // ToList req for EF
                ent.Meals = await mcx.Meals
                    .Where(o => input.MealsIds.Contains(o.Id)).ToListAsync();

                ent.BonusMeal = await mcx.FindAsync<Meal>(input.BonusMealId);
                ent.Organic = input.Organic ?? false;

                if (isCreate)
                {
                    mcx.Add(ent);
                }

                await mcx.SaveChangesAsync();

                return Json(new { Item = MapToGridModel(ent) });
            }

            return Json(ModelState.GetErrorsInline());
        }

        public async Task<IActionResult> Delete(int id)
        {
            var dinner = await mcx.FindAsync<Dinner>(id);

            return PartialView(new DeleteConfirmInput
            {
                Id = id,
                Type = "dinner",
                Name = dinner.Name
            });
        }

        [HttpPost]
        public async Task<IActionResult> Delete(DeleteConfirmInput input)
        {
            mcx.Remove(await mcx.FindAsync<Dinner>(input.Id));

            await mcx.SaveChangesAsync();

            // the PopupForm's success function aweUtils.itemDeleted expects an obj with "Id" property
            return Json(new { input.Id });
        }
    }
}