using Omu.Awem.Utils;
using Omu.AwesomeMvc;
using System.Collections.Generic;

namespace Omu.Awem.Helpers
{
    /// <summary>
    /// Grid movable rows options
    /// </summary>
    public class MoveRowsOpt
    {
        /// <summary>
        /// grid ids
        /// </summary>
        public string[] Ids { get; set; }

        /// <summary>
        /// Selector for the drag handle
        /// </summary>
        public string HandleSelector { get; set; }

        /// <summary>
        /// Do not apply context prefix on the ids
        /// </summary>
        public bool NoContext { get; set; }
    }

    /// <summary>
    /// Grid inline edit options
    /// </summary>
    public class InlEditOpt
    {
        /// <summary>
        /// Url where the data will posted on save, used for both create and edit if editUrl is not specified
        /// </summary>
        public string SaveUrl { get; set; }

        /// <summary>
        /// edit save url
        /// </summary>
        public string EditUrl { get; set; }

        /// <summary>
        /// inline edit one row at a time
        /// </summary>
        public bool OneRow { get; set; }

        /// <summary>
        /// reload the grid on save (true by default);
        /// when false, the create post action on success should return the grid row model, example: Json(new { Item = MapToGridModel(newEntity) })
        /// </summary>                
        public bool? ReloadOnSave { get; set; }

        /// <summary>
        /// edit row on click
        /// </summary>
        public bool? RowClickEdit { get; set; }

        /// <summary>
        /// js object containing client options: save, edit { save: function }
        /// </summary>
        public string ClientOpt { get; set; }

        /// <summary>
        /// Batch save
        /// </summary>
        public bool Batch { get; set; }

        /// <summary>
        /// Rows are always in edit mode
        /// </summary>
        public bool AlwaysEdit { get; set; }

        /// <summary>
        /// Stop grid loading when there are unsaved changes
        /// </summary>
        public bool? StopLoad { get; set; }
    }

    /// <summary>
    /// Grid inline edit extensions
    /// </summary>
    public static class GridInlineExtensions
    {
        /// <summary>
        /// Grid move rows extension
        /// </summary>    
        public static Grid<T> MovableRows<T>(this Grid<T> grid, MoveRowsOpt opt = null)
        {
            opt = opt ?? new MoveRowsOpt();

            IEnumerable<string> resIds = opt.Ids;

            if (!opt.NoContext && resIds != null)
            {
                var ids = new List<string>();

                foreach (var id in resIds)
                {
                    ids.Add(AweUtil.GetContextPrefix(grid.Awe.Html) + id);
                }

                resIds = ids;
            }

            var mod = new GMod
            {
                Name = "awem.gridMovRows",
                Opt = new
                {
                    G = resIds,
                    handle = opt.HandleSelector
                }
            };

            grid.Mod(mod);

            return grid;
        }

        /// <summary>
        /// Inline editing
        /// </summary>
        public static Grid<T> InlEdit<T>(this Grid<T> grid, InlEditOpt opt)
        {
            if (opt.Batch && opt.RowClickEdit == null)
            {
                opt.RowClickEdit = true;
            }

            var mod = new GMod
            {
                Name = "awem.gridInlineEdit",
                Opt = new
                {
                    createUrl = opt.SaveUrl,
                    editUrl = opt.EditUrl,
                    oneRow = opt.OneRow,
                    relOnSav = opt.ReloadOnSave,
                    batch = opt.Batch,
                    rowClickEdit = opt.RowClickEdit,
                    clientSave = opt.ClientOpt,
                    alwaysEdit = opt.AlwaysEdit,
                    stopLoad = opt.StopLoad,
                }
            };

            grid.Mod(mod);

            return grid;
        }

        /// <summary>
        /// Inline id (key) hidden input
        /// </summary>        
        public static Column InlHiddenId(this Column column, string valProp = null)
        {
            return column.Mod(o => o.InlineId(valProp, hidden: true));
        }

        /// <summary>
        /// Inline id (key) hidden input
        /// </summary>        
        public static Column InlId(this Column column, string valProp = null)
        {
            return column.Mod(o => o.InlineId(valProp));
        }

        /// <summary>
        /// Inline hidden input
        /// </summary>        
        public static Column InlHidden(this Column column, string valProp = null)
        {
            return column.Mod(o => o.InlineHidden(valProp));
        }

        /// <summary>
        /// Inline readonly
        /// </summary>        
        public static Column InlReadonly(this Column column)
        {
            return column.Mod(o => o.InlineReadonly());
        }

        /// <summary>
        /// Inline awesome helper
        /// </summary>
        /// <param name="column"></param>
        /// <param name="helper"></param>
        /// <param name="valProp"></param>
        /// <returns></returns>
        public static Column Inl(this Column column, IAwePrefHelper helper, string valProp = null)
        {
            return column.Mod(columnCfg =>
            {
                columnCfg.AddInlElmFunc(cx =>
                {
                    return InlHelperUtil.GetFilterInlElm(helper);
                });
            });
        }

        /// <summary>
        /// Inline format
        /// </summary>
        /// <param name="column"></param>
        /// <param name="format"></param>
        /// <param name="valProp"></param>
        /// <returns></returns>
        public static Column Inl(this Column column, string format, string valProp = null)
        {
            return column.Mod(o => o.Inline(format, valProp));
        }

        /// <summary>
        /// Inline textbox
        /// </summary>
        public static Column InlString(this Column column, TextBoxOpt opt = null)
        {
            opt = opt ?? new TextBoxOpt();
            var name = column.Bind;

            opt.Name = opt.Name ?? name;

            if (opt.Placeholder == null)
            {
                opt.Placeholder = InternalUtils.SplitByCapLetter(name);
            }

            column.Mod(columnCfg =>
            {
                columnCfg.AddInlElmFunc(cx =>
                {
                    return InlHelperUtil.GetFilterInlElm(cx.Html.Awe().TextBox(opt));
                });
            });

            return column;
        }

        /// <summary>
        /// Inline datepicker
        /// </summary>
        /// <param name="column"></param>
        /// <returns></returns>
        public static Column InlDate(this Column column)
        {
            var name = column.Bind;

            column.Mod(columnCfg =>
            {
                columnCfg.AddInlElmFunc(cx =>
                {
                    return InlHelperUtil.GetFilterInlElm(
                        cx.Html.Awe().DatePicker(name).ChangeYear().ChangeMonth()
                        );
                });
            });

            return column;
        }

        /// <summary>
        /// Inline DropdownList
        /// </summary>
        /// <param name="column"></param>
        /// <param name="opt"></param>
        /// <returns></returns>
        public static Column InlDropdownList(this Column column, DropdownListOpt opt)
        {
            var name = column.Bind;

            opt.Name = opt.Name ?? name;

            column.Mod(columnCfg =>
            {
                columnCfg.AddInlElmFunc(cx =>
                {
                    return InlHelperUtil.GetFilterInlElm(cx.Html.Awe().DropdownList(opt));
                });
            });

            return column;
        }

        /// <summary>
        /// Inline Multiselect
        /// </summary>
        /// <param name="column"></param>
        /// <param name="opt"></param>
        /// <returns></returns>
        public static Column InlMultiselect(this Column column, MultiselectOpt opt)
        {
            var name = column.Bind;

            opt.Name = opt.Name ?? name;

            column.Mod(columnCfg =>
            {
                columnCfg.AddInlElmFunc(cx =>
                {
                    return InlHelperUtil.GetFilterInlElm(cx.Html.Awe().Multiselect(opt));
                });
            });

            return column;
        }
    }
}