using Omu.AwesomeMvc;
using System;

namespace Omu.Awem.Helpers
{
    internal static class InlHelperUtil
    {
        public static FilterInlElm GetFilterInlElm(IAwePrefHelper helper, string valProp = null)
        {
            helper.Pref = "#Prefix";

            var valHelper = helper as IAweValHelper;
            if (valHelper != null)
            {
                valHelper.Sval = "#Value";
            }

            var formats = GetFormats(helper);
            var prop = valProp ?? helper.Awe.Prop;
            return new FilterInlElm { format = formats.Item1, strFunc = formats.Item2, strOpt = formats.Item3, valProp = prop, modelProp = prop };
        }

        public static Tuple<string, string, string> GetFormats(IAwePrefHelper helper)
        {
            var partRender = helper as IPartRenderBase;
            if (partRender != null)
            {
                var parts = partRender.GetParts();
                return new Tuple<string, string, string>(parts[0], parts[1], parts[2]);
            }
            else
            {
                return new Tuple<string, string, string>(helper.ToString(), null, null);
            }
        }
    }

    /// <summary>
    /// 
    /// </summary>
    public static class InlineEditExtensions
    {
        /// <summary>
        /// Inline hidden input for Id column, will use "Id" as name if bind or valprop is not specified
        /// </summary>
        /// <param name="builder"></param>
        /// <param name="valProp">grid model property to get value from, default Column.Bind</param>
        /// <param name="modelProp">viewmodel property to match in the edit/create action, when not set valProp will be used</param>
        /// <param name="hidden">no display value</param>
        /// <returns></returns>
        public static T InlineId<T>(this T builder, string valProp = null, string modelProp = null, bool hidden = false) where T : IInlColBuilder
        {
            valProp = valProp ?? builder.Column.Bind ?? "Id";
            modelProp = modelProp ?? valProp;

            var format = "<input id='#Prefix" + modelProp + "' type='hidden' name='" + modelProp + "' value='#Value'>";

            if (!hidden)
            {
                format += "#Value";
            }

            setFormat(builder, format, valProp, modelProp);
            return builder;
        }

        /// <summary>
        /// Inline hidden input
        /// </summary>
        /// <param name="builder"></param>
        /// <param name="valProp">grid model property to get value from, when not set will use Column.Bind</param>
        /// <param name="modelProp">viewmodel property to match in the edit/create action, when not set valProp will be used</param>        
        public static T InlineHidden<T>(this T builder, string valProp = null, string modelProp = null) where T : IInlColBuilder
        {
            valProp = valProp ?? builder.Column.Bind;
            modelProp = modelProp ?? valProp;
            var format = "<input id='#Prefix" + modelProp + "' type='hidden' name='" + modelProp + "' value='#Value'>";
            setFormat(builder, format, valProp, modelProp);
            return builder;
        }

        /// <summary>
        /// add inline awesome helper
        /// </summary>
        public static T Inline<T>(this T builder, IAwePrefHelper helper, string valProp = null) where T : IInlColBuilder
        {
            helper.Pref = "#Prefix";

            var valHelper = helper as IAweValHelper;
            if (valHelper != null)
            {
                valHelper.Sval = "#Value";
            }

            setFormat(builder, helper, valProp);

            return builder;
        }

        /// <summary>
        /// set filter format
        /// </summary>
        /// <typeparam name="T"></typeparam>        
        /// <param name="builder"></param>
        /// <param name="helper"></param>
        /// <param name="valProp">filter model property name, by default awe helper name is used</param>
        /// <returns></returns>
        public static T Filter<T>(this T builder, IAweValHelper helper, string valProp = null) where T : IInlColBuilder
        {
            helper.Sval = "#Value";
            helper.Pref = "#Prefix";

            var formats = InlHelperUtil.GetFormats(helper);

            builder.AddFilter(new FilterInlElm { format = formats.Item1, strFunc = formats.Item2, valProp = valProp ?? helper.Awe.Prop, strOpt = formats.Item3 });

            return builder;
        }

        /// <summary>
        /// Set filter format for the column
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="builder"></param>
        /// <param name="format">html format</param>
        /// <param name="jsFunc">js format</param>
        /// <param name="jsFuncArg">js function argument</param>
        /// <param name="clearBtn"></param>
        /// <param name="valProp">filter model property name, by default Column.Bind is used</param>
        /// <param name="type"></param>
        /// <returns></returns>
        public static T Filter<T>(this T builder,
            string format = null,
            string jsFunc = null,
            string jsFuncArg = null,
            bool? clearBtn = null,
            string valProp = null,
            FilterType type = FilterType.Default)
            where T : IInlColBuilder
        {
            string stype = null;
            if (type == FilterType.Multiselect)
            {
                stype = "multiselect";
            }
            else if (type == FilterType.Multichk)
            {
                stype = "multichk";
            }

            builder.AddFilter(new FilterInlElm
            {
                valProp = valProp,
                type = stype,
                format = format,
                strFunc = jsFunc,
                strOpt = jsFuncArg,
                clearBtn = clearBtn
            });

            return builder;
        }

        /// <summary>
        /// set inline format for the column; value from valProp is used to replace #Value in the format
        /// </summary>
        /// <param name="builder"></param>
        /// <param name="format">editor string format, #Value and #Prefix will be replaced, prefix is used for unique ids </param>
        /// <param name="valProp">grid model property to get value from, when not set will use Column.Bind</param>
        /// <param name="modelProp">viewmodel property to match in the edit/create action, when not set valProp will be used, used for validation only here, you set the input name in the string format</param>
        /// <returns></returns>
        public static T Inline<T>(this T builder, string format, string valProp = null, string modelProp = null) where T : IInlColBuilder
        {
            setFormat(builder, format, valProp, modelProp);
            return builder;
        }

        /// <summary>
        /// set readonly inline format for the column
        /// </summary>
        /// <param name="builder"></param>
        /// <param name="displayFormat">display format, .Prop will be replaced with values from the grid row model </param>
        /// <param name="valProp">grid model property to get value from, when not set will use Column.Bind</param>
        /// <param name="modelProp">viewmodel property to match in the edit/create action, when not set valProp will be used, used for validation only here, you set the input name in the string format</param>
        /// <returns></returns>
        public static T InlineReadonly<T>(this T builder, string displayFormat = null, string valProp = null, string modelProp = null) where T : IInlColBuilder
        {
            valProp = valProp ?? builder.Column.Bind;
            modelProp = modelProp ?? valProp;

            displayFormat = displayFormat ?? builder.Column.ClientFormat ?? "#Value";

            var format = displayFormat + "<input type='hidden' name='" + modelProp + "' value='#Value'/>";

            setFormat(builder, format, valProp, modelProp);

            return builder;
        }

        /// <summary>
        /// Inline checkbox, by default will use column.Bind + "Checked" for valueProp and column.Bind for modelProp
        /// </summary>
        /// <param name="builder"></param>
        /// <param name="valProp">grid model property to get value from, when not set will use Column.Bind</param>
        /// <param name="modelProp">viewmodel property to match in the edit/create action, when not set valProp will be used</param>
        /// <param name="cssClass">css class for the checkbox input</param>
        /// <returns></returns>
        public static T InlineBool<T>(this T builder, string valProp = null, string modelProp = null, string cssClass = "") where T : IInlColBuilder
        {
            valProp = valProp ?? builder.Column.Bind;
            modelProp = modelProp ?? valProp;

            var format = "<input type='checkbox' name='" + modelProp + "' value='true' #ValChecked class='" + cssClass + "' />"; // #ValChecked will be replaced by checked or nothing
            setFormat(builder, format, valProp, modelProp);

            return builder;
        }

        private static void setFormat(IInlColBuilder builder, string format, string valProp, string modelProp = null, string func = null, string funcArg = null)
        {
            valProp = valProp ?? builder.Column.Bind;
            modelProp = modelProp ?? valProp;

            builder.AddInline(new InlElm { format = format, modelProp = modelProp, valProp = valProp, strFunc = func, strOpt = funcArg });
        }

        //private static void setFormat(IInlColBuilder builder, IAweValHelper helper, string valProp, string modelProp = null)
        //{
        //    var formats = InlHelperUtil.GetFormats(helper);

        //    setFormat(builder, formats.Item1, valProp ?? helper.Awe.Prop, modelProp ?? helper.Awe.Prop, formats.Item2, formats.Item3);
        //}

        private static void setFormat(IInlColBuilder builder, IAwePrefHelper helper, string valProp, string modelProp = null)
        {
            var formats = InlHelperUtil.GetFormats(helper);

            setFormat(builder, formats.Item1, valProp ?? helper.Awe.Prop ?? helper.Name, modelProp ?? helper.Awe.Prop, formats.Item2, formats.Item3);
        }
    }
}