using Omu.AwesomeMvc;
using System.Collections.Generic;
using System.Linq;

namespace Omu.Awem.Helpers
{
    /// <summary>
    /// Awesome Controls Opt Extensions
    /// </summary>
    public static class OptExtensions
    {
        /// <summary>
        /// DropdownList Fav buttons
        /// </summary>
        /// <param name="opt"></param>
        /// <param name="count"></param>
        /// <param name="values"></param>
        /// <param name="frozen"></param>
        /// <returns></returns>
        public static DropdownListOpt Favs(this DropdownListOpt opt, int count = 3, object[] values = null, object[] frozen = null)
        {
            opt.Favs = new FavOpt
            {
                Count = count,
                Values = values,
                Frozen = frozen
            };

            return opt;
        }

        /// <summary>
        /// Add a Parent element
        /// On load the value of the parent is sent to the server
        /// </summary>
        /// <param name="opt"></param>
        /// <param name="id"> the html id of the parent element</param>
        /// <param name="alias"> the alias to be used at the server side for the parent (default = 'parent')</param>
        /// <param name="loadOnChange">reload when parent changes its value</param>
        /// <param name="context">add awe context prefix to the parent id</param>                         
        public static T Parent<T>(this T opt, string id, string alias = "parent", bool loadOnChange = true, bool context = true) where T : IParentParamOpt
        {
            if (opt.Parents == null) opt.Parents = new List<ParentOpt>();

            opt.Parents.Add(new ParentOpt { Id = id, Alias = alias, LoadOnChange = loadOnChange, Context = context });

            return opt;
        }

        /// <summary>
        /// Add popup button
        /// </summary>
        /// <param name="opt"></param>
        /// <param name="text"></param>
        /// <param name="action"></param>
        /// <returns></returns>
        public static T Button<T>(this T opt, string text, string action) where T : PopupBaseOpt
        {
            var btns = (opt.Buttons ?? new List<PopupButtonOpt>()).ToList();

            btns.Add(new PopupButtonOpt { Text = text, Action = action });

            opt.Buttons = btns;
            return opt;
        }

        /// <summary>
        /// Set parent for grid inline editing editor
        /// </summary>
        public static T ParentInl<T>(this T opt, string id, string alias = "parent", bool loadOnChange = true) where T : IParentParamOpt
        {
            opt.Parent("#Prefix" + id, alias, loadOnChange);

            return opt;
        }

        /// <summary>
        /// Set a parameter.
        /// On load parameters are sent to the server.
        /// </summary>
        /// <param name="opt"></param>
        /// <param name="name">the name of the parameter to be used on the server side</param>
        /// <param name="value">the value of the parameter</param>
        /// <returns></returns>
        public static T Parameter<T>(this T opt, string name, object value) where T : IParentParamOpt
        {
            if (opt.Parameters == null) opt.Parameters = new List<ParameterOpt>();

            opt.Parameters.Add(new ParameterOpt { Name = name, Value = value });

            return opt;
        }

        /// <summary>
        /// FilterRow editor won't reload on each grid load (page, sort, etc.)
        /// </summary>
        public static T FilterNoReload<T>(this T opt) where T : DataEditorOpt
        {
            opt.CssClass = (opt.CssClass ?? string.Empty) + " o-nfload"; // editor with this class won't be reloaded on grid load            
            return opt;
        }

        /// <summary>
        /// Remote search
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="opt"></param>
        /// <param name="url">remote search url</param>
        /// <param name="key">cache key</param>
        /// <param name="noCache">don't use cache</param>
        /// <returns></returns>
        public static T RemoteSearch<T>(this T opt, string url, string key = null, bool noCache = false) where T : DropdownOpt
        {
            opt.SearchFunc = new SearchFuncOpt
            {
                Name = "aweUtils.osearch",
                Url = url,
                Key = key,
                NoCache = noCache
            };

            return opt;
        }

        /// <summary>
        /// Sets filter datafunc to get filter data (if opt.DataFunc and Url is not already set) 
        /// and filter parameterFunc to send filter parameters, also clear button true
        /// </summary>
        /// <param name="opt"></param>
        /// <param name="name"></param>
        /// <returns></returns>
        public static T FilterData<T>(this T opt, string name = null) where T : DataEditorOpt
        {
            name = name ?? opt.Name;

            if (opt.DataFunc == null && opt.Url == null)
            {
                opt.DataFunc = "aweUtils.frowData('" + name + "')";
            }

            opt.ParameterFunc = opt.ParameterFunc ?? "aweUtils.getFparams";
            return opt;
        }

        /// <summary>
        /// Instant filter (set o-instch css class)        
        /// </summary>
        public static TextBoxOpt FltInstant(this TextBoxOpt opt)
        {
            opt.CssClass = (opt.CssClass ?? string.Empty) + " o-instch";
            return opt;
        }
    }
}