using System;
using System.Collections.Generic;
using System.Linq.Expressions;
using System.Text.RegularExpressions;

namespace Omu.Awem.Utils
{
    /// <summary>
    /// 
    /// </summary>
    public static class AweExprUtil
    {
        internal static MemberExpression GetBindMemberExpression<TModel>(ParameterExpression itmParam, string bind)
        {
            MemberExpression propAccess = null;
            if (bind.Contains("."))
            {
                var memberNames = bind.Split('.');

                var memberType = typeof(TModel);
                for (var i = 0; i < memberNames.Length; i++)
                {
                    var property = memberType.GetProperty(memberNames[i]);
                    memberType = property.PropertyType;

                    if (propAccess != null)
                    {
                        propAccess = Expression.MakeMemberAccess(propAccess, property);
                    }
                    else
                    {
                        propAccess = Expression.MakeMemberAccess(itmParam, property);
                    }
                }
            }
            else
            {
                propAccess = Expression.Property(itmParam, typeof(TModel).GetProperty(bind));
            }

            return propAccess;
        }

        /// <summary>
        /// Build contains expression
        /// </summary>
        /// <typeparam name="TModel"></typeparam>
        /// <param name="val"></param>
        /// <param name="bind"></param>
        /// <returns></returns>
        public static Expression<Func<TModel, bool>> ContainsExpr<TModel>(string val, string bind)
        {
            var lambdaParam = Expression.Parameter(typeof(TModel));

            MemberExpression propAccess = AweExprUtil.GetBindMemberExpression<TModel>(lambdaParam, bind);

            return MakeContains<TModel>(new[] { lambdaParam }, Expression.Constant(val, typeof(string)), propAccess);
        }

        /// <summary>
        /// Build a contains expression given the value and properties names, ( o => o.Prop1.Contains(val) || o.Prop2.Contains(val) ... )
        /// </summary>
        /// <typeparam name="TModel"></typeparam>
        /// <param name="val"></param>
        /// <param name="propExprs"></param>
        /// <returns></returns>
        public static Expression<Func<TModel, bool>> ContainsExpr<TModel>(string val, Expression<Func<TModel, string>>[] propExprs)
        {
            Expression<Func<TModel, bool>> res = null;
            foreach (var propExpr in propExprs)
            {
                var propAcc = propExpr.Body as MemberExpression;

                var contLambda = MakeContains<TModel>(propExpr.Parameters, Expression.Constant(val, typeof(string)), propAcc);

                if (res == null)
                {
                    res = contLambda;
                }
                else
                {
                    res = res.Or(contLambda);
                }
            }

            return res;
        }

        /// <summary>
        /// Build a contains expression given the keywords and propertyExpr, ( o => o.Prop.Contains(val) and o.Prop.Contains(val) ... )
        /// </summary>
        public static Expression<Func<TModel, bool>> ContainsWordsAndExpr<TModel>(string[] keywords, Expression<Func<TModel, string>> propExpr)
        {
            Expression<Func<TModel, bool>> res = null;

            foreach (var keyword in keywords)
            {
                var propAcc = propExpr.Body as MemberExpression;
                var contLambda = MakeContains<TModel>(propExpr.Parameters, Expression.Constant(keyword, typeof(string)), propAcc);
                if (res == null)
                {
                    res = contLambda;
                }
                else
                {
                    res = res.And(contLambda);
                }
            }

            return res;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <typeparam name="TModel"></typeparam>
        /// <param name="val"></param>
        /// <param name="propExprs"></param>
        /// <returns></returns>
        public static Expression<Func<TModel, bool>> IndexOf<TModel>(object val, Expression<Func<TModel, string>>[] propExprs)
        {
            Expression<Func<TModel, bool>> res = null;
            foreach (var propExpr in propExprs)
            {
                var propAcc = propExpr.Body as MemberExpression;

                var contLambda = MakeIndexOf<TModel>(propExpr.Parameters, Expression.Constant((string)val, typeof(string)), propAcc);

                if (res == null)
                {
                    res = contLambda;
                }
                else
                {
                    res = res.Or(contLambda);
                }
            }

            return res;
        }

        private static Expression<Func<TModel, bool>> MakeContains<TModel>(
            IEnumerable<ParameterExpression> parameters, // lambda param
            Expression argument, // contains method arguments
            MemberExpression prop)
        {
            var containsMethodInfo = typeof(string).GetMethod("Contains", new[] { typeof(string) });

            var mainExpr = Expression.Call(prop, containsMethodInfo, argument);

            return Expression.Lambda<Func<TModel, bool>>(mainExpr, parameters);
        }

        /// <summary>
        /// Create indexOf expression
        /// </summary>
        public static Expression<Func<TModel, bool>> IndexOf<TModel>(object val, string bind)
        {
            var lambdaParam = Expression.Parameter(typeof(TModel));

            MemberExpression propAccess = AweExprUtil.GetBindMemberExpression<TModel>(lambdaParam, bind);

            return MakeIndexOf<TModel>(new[] { lambdaParam }, Expression.Constant((string)val, typeof(string)), propAccess);
        }

        private static Expression<Func<TModel, bool>> MakeIndexOf<TModel>(
            IEnumerable<ParameterExpression> parameters,
            Expression argument,
            MemberExpression prop)
        {
            var indexOfMet = typeof(string).GetMethod(
                "IndexOf",
                new[] { typeof(string), typeof(StringComparison) });

            var indexOfCallArgs = new[] {
                argument,
                Expression.Constant(StringComparison.InvariantCultureIgnoreCase)};

            var indexOfCall = Expression.Call(prop, indexOfMet, indexOfCallArgs);

            var greatCall = Expression.GreaterThan(indexOfCall, Expression.Constant(-1));

            return Expression.Lambda<Func<TModel, bool>>(greatCall, parameters);
        }
    }

    static class Extensions
    {
        public static Expression<Func<T, bool>> Or<T>(
            this Expression<Func<T, bool>> expr1,
            Expression<Func<T, bool>> expr2)
        {
            var parameter = Expression.Parameter(typeof(T));

            var leftVisitor = new ReplaceExpressionVisitor(expr1.Parameters[0], parameter);
            var left = leftVisitor.Visit(expr1.Body);

            var rightVisitor = new ReplaceExpressionVisitor(expr2.Parameters[0], parameter);
            var right = rightVisitor.Visit(expr2.Body);

            return Expression.Lambda<Func<T, bool>>(
                Expression.Or(left, right), parameter);
        }

        public static Expression<Func<T, bool>> And<T>(
           this Expression<Func<T, bool>> expr1,
           Expression<Func<T, bool>> expr2)
        {
            var parameter = Expression.Parameter(typeof(T));

            var leftVisitor = new ReplaceExpressionVisitor(expr1.Parameters[0], parameter);
            var left = leftVisitor.Visit(expr1.Body);

            var rightVisitor = new ReplaceExpressionVisitor(expr2.Parameters[0], parameter);
            var right = rightVisitor.Visit(expr2.Body);

            return Expression.Lambda<Func<T, bool>>(
                Expression.And(left, right), parameter);
        }
    }

    class ReplaceExpressionVisitor : ExpressionVisitor
    {
        private readonly Expression _oldValue;
        private readonly Expression _newValue;

        public ReplaceExpressionVisitor(Expression oldValue, Expression newValue)
        {
            _oldValue = oldValue;
            _newValue = newValue;
        }

        public override Expression Visit(Expression node)
        {
            if (node == _oldValue)
                return _newValue;
            return base.Visit(node);
        }
    }

    internal static class InternalUtils
    {
        /// <summary>
        /// from HiThere to Hi There
        /// </summary>
        public static string SplitByCapLetter(string s)
        {
            var r = new Regex(@"
                (?<=[A-Z])(?=[A-Z][a-z]) |
                 (?<=[^A-Z])(?=[A-Z]) |
                 (?<=[A-Za-z])(?=[^A-Za-z])", RegexOptions.IgnorePatternWhitespace);

            return r.Replace(s, " ");
        }
    }
}