using Omu.AwesomeMvc;
using System;
using System.Linq.Expressions;
using System.Reflection;

namespace Omu.Awem.Utils
{
    /// <summary>
    /// Column filter utils
    /// </summary>
    public static class ColumnFilterUtil
    {
        /// <summary>
        /// String filter
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="fb"></param>
        /// <param name="propName">property to filter by</param>
        /// <returns></returns>
        public static FilterBuilder<T> StringFilter<T>(this FilterBuilder<T> fb, string propName)
        {
            fb.Add(propName, new FilterRule<T>
            {
                QueryValFunc = StringQuery<T>(propName)
            });

            return fb;
        }

        /// <summary>
        /// Filter for a DateTime model property, with an operation (= &lt; &gt;) 
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="fb"></param>
        /// <param name="propName">property to filter by</param>
        /// <returns></returns>
        public static FilterBuilder<T> DateOpFilter<T>(this FilterBuilder<T> fb, string propName)
        {
            var opValue = fb.Filter.GetType().GetProperty(propName + "Op").GetValue(fb.Filter) as string;

            fb.Add(propName, new FilterRule<T>
            {
                QueryValFunc = ColumnFilterUtil.OpQuery<T, DateTime>(propName, opValue ?? "=")
            });

            return fb;
        }

        /// <summary>
        /// Filter for an int property with an operation (= &lt; &gt;)
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="fb"></param>
        /// <param name="propName"></param>
        /// <returns></returns>
        public static FilterBuilder<T> NumericOpFilter<T>(this FilterBuilder<T> fb, string propName)
        {
            var opProp = getProperty(getOpPropName(propName), fb);
            var opValue = opProp.GetValue(fb.Filter) as string;

            fb.Add(propName, new FilterRule<T>
            {
                QueryValFunc = ColumnFilterUtil.OpQuery<T, int>(propName, opValue ?? "=")
            });

            return fb;
        }

        private static string getOpPropName(string propName)
        {
            return propName + "Op";
        }

        private static PropertyInfo getProperty<T>(string propName, FilterBuilder<T> fb)
        {
            var prop = fb.Filter.GetType().GetProperty(propName);

            if (prop == null)
            {
                throw new ArgumentNullException(propName, "property " + propName + " not found on the filter model");
            }

            return prop;
        }

        /// <summary>
        /// Filter for a property with an operation (= &lt; &gt;);
        /// The filter model should also have a property with the same name as the property passed to this method + "Op" suffix
        /// </summary>
        public static FilterBuilder<T> OpFilter<T, TModelPropertyType>(this FilterBuilder<T> fb, string propName)
        {
            var prop = getProperty(propName, fb);

            var opPropName = propName + "Op";
            var opProp = getProperty(opPropName, fb);

            var opValue = opProp.GetValue(fb.Filter) as string;

            fb.Add(propName, new FilterRule<T>
            {
                QueryValFunc = val => OpQueryExpression<T, TModelPropertyType>(val, propName, opValue ?? "=")
            });

            return fb;
        }

        /// <summary>
        /// Get string query expression for grid
        /// </summary>
        public static Func<object, Expression<Func<TModel, bool>>> StringQuery<TModel>(string propName)
        {
            if (Settings.StringFilterIgnoreCase)
            {
                return val => AweExprUtil.IndexOf<TModel>(val, propName);
            }

            return val => AweExprUtil.ContainsExpr<TModel>((string)val, propName);
        }

        /// <summary>
        /// Get Op Query expression for grid
        /// </summary>        
        public static Func<object, Expression<Func<TModel, bool>>> OpQuery<TModel, TValue>(string propName, string opValue)
        {
            return val => OpQueryExpression<TModel, TValue>(val, propName, opValue);
        }

        private static Expression<Func<TModel, bool>> OpQueryExpression<TModel>(object val, string propName, Type tvalue, string opValue = "=")
        {
            var pval = Convert.ChangeType(val, tvalue);
            var opKey = propName + "_Op";

            var itmParam = Expression.Parameter(typeof(TModel));

            var valPropExpr = AweExprUtil.GetBindMemberExpression<TModel>(itmParam, propName);
            var valConstExpr = Expression.Constant(pval, tvalue);

            if (opValue == "<")
            {
                return Expression.Lambda<Func<TModel, bool>>(Expression.LessThan(valPropExpr, valConstExpr), itmParam);
            }

            if (opValue == "<=")
            {
                return Expression.Lambda<Func<TModel, bool>>(Expression.LessThanOrEqual(valPropExpr, valConstExpr), itmParam);
            }

            if (opValue == ">")
            {
                return Expression.Lambda<Func<TModel, bool>>(Expression.GreaterThan(valPropExpr, valConstExpr), itmParam);
            }

            if (opValue == ">=")
            {
                return Expression.Lambda<Func<TModel, bool>>(Expression.GreaterThanOrEqual(valPropExpr, valConstExpr), itmParam);
            }

            if (opValue == "!=")
            {
                return Expression.Lambda<Func<TModel, bool>>(Expression.NotEqual(valPropExpr, valConstExpr), itmParam);
            }

            if (opValue == "=")
            {
                return Expression.Lambda<Func<TModel, bool>>(Expression.Equal(valPropExpr, valConstExpr), itmParam);
            }

            throw new ArgumentException("uknown filter operator [" + opValue + "]");
        }

        private static Expression<Func<TModel, bool>> OpQueryExpression<TModel, TValue>(object val, string propName, string opValue = "=")
        {
            var pval = (TValue)val;
            var opKey = propName + "_Op";

            var itmParam = Expression.Parameter(typeof(TModel));

            var valPropExpr = AweExprUtil.GetBindMemberExpression<TModel>(itmParam, propName);
            var valConstExpr = Expression.Constant(pval, typeof(TValue));

            if (opValue == "<")
            {
                return Expression.Lambda<Func<TModel, bool>>(Expression.LessThan(valPropExpr, valConstExpr), itmParam);
            }

            if (opValue == "<=")
            {
                return Expression.Lambda<Func<TModel, bool>>(Expression.LessThanOrEqual(valPropExpr, valConstExpr), itmParam);
            }

            if (opValue == ">")
            {
                return Expression.Lambda<Func<TModel, bool>>(Expression.GreaterThan(valPropExpr, valConstExpr), itmParam);
            }

            if (opValue == ">=")
            {
                return Expression.Lambda<Func<TModel, bool>>(Expression.GreaterThanOrEqual(valPropExpr, valConstExpr), itmParam);
            }

            if (opValue == "!=")
            {
                return Expression.Lambda<Func<TModel, bool>>(Expression.NotEqual(valPropExpr, valConstExpr), itmParam);
            }

            if (opValue == "=")
            {
                return Expression.Lambda<Func<TModel, bool>>(Expression.Equal(valPropExpr, valConstExpr), itmParam);
            }

            throw new ArgumentException("uknown filter operator [" + opValue + "]");
        }
    }
}