using System.Web.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Threading.Tasks;

namespace Omu.Awem.Utils
{
    /// <summary>
    /// controller extensions utilities methods
    /// </summary>
    public static class ControllerExtensions
    {
        /// <summary>
        /// get modelstate errors in the format required for the inline edit grid mod
        /// </summary>
        /// <param name="modelState"></param>
        /// <returns></returns>
        public static object GetErrorsInline(this ModelStateDictionary modelState)
        {
            var errorList = modelState.Where(o => o.Value.Errors.Count > 0)
                                .ToDictionary(
                                kvp => kvp.Key,
                                kvp => kvp.Value.Errors.Select(e => e.ErrorMessage).ToArray());

            return new { _errs = errorList };
        }
    }

    /// <summary>
    /// Filter rule
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class FilterRule<T>
    {
        /// <summary>
        /// Query
        /// </summary>
        public Expression<Func<T, bool>> Query { get; set; }

        /// <summary>
        /// Func that returns Query
        /// </summary>
        public Func<Expression<Func<T, bool>>> QueryFunc { get; set; }

        /// <summary>
        /// Func that receives filter value and returns Query
        /// </summary>
        public Func<object, Expression<Func<T, bool>>> QueryValFunc { get; set; }

        /// <summary>
        /// Func that returns data for the filter editor
        /// </summary>
        public Func<IQueryable<T>, Task<object>> GetData { get; set; }

        /// <summary>
        /// Get data after the Query function has been applied to the Query
        /// </summary>
        public bool SelfQuery { get; set; }
    }

    /// <summary>
    /// Filter parameter base class
    /// </summary>
    public class FilterPrm
    {
        /// <summary>
        /// Filter properties order
        /// </summary>
        public string[] Forder { get; set; }
    }

    /// <summary>
    /// Grid filter builder
    /// </summary>
    /// <typeparam name="TEntity"></typeparam>
    public class FilterBuilder<TEntity>
    {
        private readonly IDictionary<string, FilterRule<TEntity>> frules = new Dictionary<string, FilterRule<TEntity>>();

        /// <summary>
        /// Add filter rule
        /// </summary>
        /// <param name="key"></param>
        /// <param name="rule"></param>
        /// <returns></returns>
        public FilterBuilder<TEntity> Add(string key, FilterRule<TEntity> rule)
        {
            frules.Add(key, rule);
            return this;
        }

        /// <summary>
        /// Filter row editors data 
        /// </summary>
        public Dictionary<string, object> Data { get; set; }

        /// <summary>
        /// Fitler parameter
        /// </summary>
        public FilterPrm Filter { get; set; }

        /// <summary>
        /// Filter query
        /// </summary>
        public IQueryable<TEntity> Query { get; set; }

        /// <summary>
        /// Get GridModelBuilder Tag object containing filter data
        /// </summary>
        /// <returns></returns>
        public object GetGridData()
        {
            return new { frow = Data };
        }

        /// <summary>
        /// Build query using filter.Forder up to given rule name
        /// </summary>
        /// <param name="name">rule name</param>
        /// <returns>resulting query</returns>
        public IQueryable<TEntity> GetQueryFor(string name)
        {
            var prm = Filter;
            var query = Query;

            var forder = prm.Forder ?? new string[] { };

            foreach (var prop in forder)
            {
                if (!frules.ContainsKey(prop)) continue;

                var rule = frules[prop];

                IQueryable<TEntity> nquery = query;

                if (name == prop && !rule.SelfQuery)
                {
                    return nquery ?? query;
                }

                nquery = incQueryForProp(prop, query);

                query = nquery ?? query;
            }

            return query;
        }

        private IQueryable<TEntity> incQueryForProp(string prop, IQueryable<TEntity> query)
        {
            var prm = Filter;
            var nquery = query;
            var rule = frules[prop];

            if (rule.Query == null && rule.QueryFunc == null && rule.QueryValFunc == null)
            {
                return query;
            }

            // if prop filter has value    
            var value = prm.GetType().GetProperty(prop).GetValue(prm);
            if (value != null)
            {
                if (rule.QueryValFunc != null)
                {
                    nquery = query.Where(rule.QueryValFunc(value));
                }
                else if (rule.QueryFunc != null)
                {
                    nquery = query.Where(rule.QueryFunc());
                }
                else if (rule.Query != null)
                {
                    nquery = query.Where(rule.Query);
                }
            }

            return nquery;
        }

        /// <summary>
        /// Apply all filter rules incrementally according to Filter.Forder to the query and set Data
        /// </summary>
        /// <returns>resulting query</returns>
        public async Task<IQueryable<TEntity>> ApplyAsync()
        {
            var prm = Filter;
            var query = Query;

            var forder = prm.Forder ?? new string[] { };
            var data = new Dictionary<string, object>();

            foreach (var prop in forder)
            {
                if (!frules.ContainsKey(prop)) continue;

                var rule = frules[prop];

                IQueryable<TEntity> nquery = null;

                nquery = incQueryForProp(prop, query);

                if (rule.GetData != null)
                {
                    data.Add(prop, await rule.GetData(rule.SelfQuery ? nquery : query));
                }

                query = nquery ?? query;
            }

            // add rest of data
            foreach (var rulekv in frules.Where(o => !forder.Contains(o.Key)))
            {
                var rule = rulekv.Value;
                if (rule.GetData != null)
                {
                    data.Add(rulekv.Key, await rule.GetData(query));
                }
            }

            Data = data;

            return query;
        }
    }
}