using System;
using System.Linq;
using AwesomeWebFormsDemo.Data;
using AwesomeWebFormsDemo.Models;
using AwesomeWebFormsDemo.ViewModels.Input;
using System.Web.Mvc;


using Omu.AwesomeWebForms;

namespace AwesomeWebFormsDemo.Controllers.Demos.Grid
{
    /// <summary>
    /// Note: If your Key Property is not "Id" (but "MyId"), you need to change the GridModelBuilder.Key = "MyId" in GridGetItems and in the view
    /// change the Bind = "MyId" (if you have an Id column), 
    /// and for the action columns (edit, delete btns) you can either specify the MyId property (e.g. GridUtils.EditFormatForGrid("DinnersGrid", "MyId"));
    /// or in MapToGridModel additionally to o.MyId add another property Id = o.MyId
    /// parameters for Edit, Delete controller Actions need to remain called "id", that's how they are set in GridUtils.cs ( "params:{{ id:" );    
    /// </summary>
    public class DinnersGridCrudController : Controller
    {
        private static object MapToGridModel(Dinner o)
        {
            return
                new
                {
                    o.Id,
                    o.Name,
                    Date = o.Date.ToShortDateString(),
                    ChefName = o.Chef.FullName,
                    Meals = string.Join(", ", o.Meals.Select(m => m.Name)),
                    BonusMeal = o.BonusMeal.Name
                };
        }

        public ActionResult GridGetItems(GridParams g, string search = "")
        {
            var query = Db.Dinners
            .Where(o => o.Name.Contains(search)).AsQueryable();

            var model = new GridModelBuilder<Dinner>(query, g)
            {
                KeyProp = o => o.Id, // needed for api select, update, tree, nesting, EF                
                Map = MapToGridModel,
            };

            return Json(model.Build());
        }

        public ActionResult Create()
        {
            // make sure to use "return PartialView" for PopupForm/Popup views 
            // this will ignore _viewstart.cshtml so that you don't use the _Layout.cshtml and reload all the scripts
            return PartialView();
        }

        [HttpPost]
        public ActionResult Create(DinnerInput input)
        {
            return save(input);
        }

        public ActionResult Edit(int id)
        {
            var dinner = Db.Dinners
                .Single(o => o.Id == id);

            var input = new DinnerInput
            {
                Id = dinner.Id,
                Name = dinner.Name,
                ChefId = dinner.Chef.Id,
                Date = dinner.Date,
                MealsIds = dinner.Meals.Select(o => o.Id),
                BonusMealId = dinner.BonusMeal.Id
            };

            return PartialView("Create", input);
        }

        [HttpPost]
        public ActionResult Edit(DinnerInput input)
        {
            return save(input);
        }

        private ActionResult save(DinnerInput input)
        {
            if (!ModelState.IsValid) return PartialView("Create", input);

            var isCreate = !input.Id.HasValue;

            var ent = isCreate ? new Dinner() :
                Db.Dinners
                    .First(o => o.Id == input.Id);

            ent.Name = input.Name;
            ent.Date = input.Date.Value;
            ent.Chef = Db.Find<Chef>(input.ChefId);

            // ToList req for EF
            ent.Meals = Db.Meals
                .Where(o => input.MealsIds.Contains(o.Id)).ToList();

            ent.BonusMeal = Db.Find<Meal>(input.BonusMealId);
            ent.Organic = input.Organic ?? false;

            if (isCreate)
            {
                Db.Add(ent);
            }

            // json obj = success for the popupform, id is used to flash the row
            return Json(new { Id = ent.Id });
        }

        public ActionResult Delete(int id)
        {
            var dinner = Db.Find<Dinner>(id);

            return PartialView(new DeleteConfirmInput
            {
                Id = id,
                Type = "dinner",
                Name = dinner.Name
            });
        }

        [HttpPost]
        public ActionResult Delete(DeleteConfirmInput input)
        {
            Db.Delete<Dinner>(input.Id);

            // delete PopupForm's success function will use the Id to animate the row
            return Json(new { Id = input.Id });
        }
    }
}