using System.IO;
using System;
using System.Collections.Generic;
using System.Linq;
using AwesomeWebFormsDemo.Data;
using AwesomeWebFormsDemo.Models;
using AwesomeWebFormsDemo.Utils;
using iText.Kernel.Pdf;
using iText.Layout;
using iText.Layout.Element;
using System.Web.Mvc;


using NPOI.HSSF.UserModel;
using Omu.AwemWebForms.Export;
using Omu.AwesomeWebForms;

namespace AwesomeWebFormsDemo.Controllers.Demos.Grid
{
    public class GridExportToExcelDemoController : Controller
    {
        public ActionResult Index()
        {
            return View();
        }

        public ActionResult GetItems(GridParams g)
        {
            return Json((BuildGridModel(g)).ToDto());
        }

        private GridModel<Lunch> BuildGridModel(GridParams g)
        {
            var query = Db.Lunches.AsQueryable();

            var gmb = new GridModelBuilder<Lunch>(query, g)
            {
                KeyProp = o => o.Id,
                Map = o => new
                {
                    o.Id,
                    o.Person,
                    FoodName = o.Food.Name,
                    o.Location,
                    o.Price,
                    Date = o.Date.ToShortDateString(),
                    CountryName = o.Country.Name,
                    ChefName = o.Chef.FullName
                },
                MakeFooter = info =>
                    {
                        if (info.Level == 0)
                        {
                            return new
                            {
                                Id = "Total",
                                Price = "Min: " + info.Items.Min(o => o.Price),
                                Date = "Max: " + info.Items.Max(o => o.Date).ToShortDateString()
                            };
                        }

                        return new
                        {
                            Price = "Min: " + info.Items.Min(o => o.Price),
                            Date = "Max: " + info.Items.Max(o => o.Date).ToShortDateString()
                        };
                    }
            };

            return gmb.BuildModel();
        }

        private IEnumerable<ExpColumn> addServerFormatting(IEnumerable<ExpColumn> columns)
        {
            var priceColumn = columns.SingleOrDefault(o => o.Name == "Price");

            if (priceColumn != null)
            {
                priceColumn.ClientFormatFunc = (lunch, property) =>
                {
                    var value = lunch.GetType().GetProperty("Price")?.GetValue(lunch).ToString();
                    if (!string.IsNullOrEmpty(value))
                    {
                        value += " USD";
                    }

                    return value;
                };

                priceColumn.Width = 200;
            }

            return columns;
        }

        [HttpPost]
        public ActionResult ExportGridToExcel(GridParams g, IEnumerable<ExpColumn> expColumns, bool? allPages)
        {
            if (allPages.HasValue && allPages.Value)
            {
                g.Paging = false;
            }

            var gridModel = BuildGridModel(g);

            var workbook = new GridExcelBuilder(expColumns.Where(o => !o.Hidden)).Build(gridModel);

            using (var stream = new MemoryStream())
            {
                workbook.Write(stream);
                stream.Close();
                return File(stream.ToArray(), "application/vnd.ms-excel", "lunches.xls");
            }
        }

        [HttpPost]
        public ActionResult ExportGridToTxt(GridParams g, IEnumerable<ExpColumn> expColumns, bool? allPages)
        {
            if (allPages.HasValue && allPages.Value)
            {
                g.Paging = false;
            }

            var gridModel = BuildGridModel(g);

            var res = new GridTxtBuilder(expColumns.Where(o => !o.Hidden)).Build(gridModel);

            using (var memoryStream = new MemoryStream())
            {
                using (var writer = new StreamWriter(memoryStream))
                {
                    writer.WriteLine(res);
                }

                return File(memoryStream.ToArray(), "text/plain", "lunches.txt");
            }
        }

        [HttpPost]
        public ActionResult ExportGridToPdf(GridParams g, IEnumerable<ExpColumn> expColumns, bool? allPages)
        {
            if (allPages.HasValue && allPages.Value)
            {
                g.Paging = false;
            }

            var gridModel = BuildGridModel(g);

            var builder = new GridPdfBuilder(addServerFormatting(expColumns.Where(o => !o.Hidden)));

            var workStream = new MemoryStream();
            var writer = new PdfWriter(workStream);
            var pdf = new PdfDocument(writer);
            var document = new Document(pdf);

            document.Add(new Paragraph("Hello World!"));
            document.Add(new Paragraph("Export to pdf example (using iText7)"));
            document.Add(new Paragraph(new Text("\n")));

            var table = builder.Build(gridModel);
            document.Add(table);
            writer.SetCloseStream(false);
            document.Close();

            var byteInfo = workStream.ToArray();
            workStream.Write(byteInfo, 0, byteInfo.Length);
            workStream.Position = 0;

            return File(workStream, "application/pdf", "lunches.pdf");
        }

        /// <summary>
        /// Demonstrates the simplest way of creating an excel workbook 
        /// it exports all the lunches records, without taking into account any sorting/paging that is done on the client side
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public ActionResult ExportAllToExcel()
        {
            var workbook = new HSSFWorkbook();
            var sheet = workbook.CreateSheet("sheet1");

            var res = Db.Lunches.ToArray();

            var items = res.Select(
                o => new
                {
                    o.Id,
                    o.Person,
                    FoodName = o.Food.Name,
                    o.Location,
                    CountryName = o.Country.Name,
                    ChefName = o.Chef.FullName
                }).ToList();

            var properties = new[] { "Id", "Person", "FoodName", "CountryName", "ChefName", "Location" };
            var headers = new[] { "Id", "Person", "Food", "Country", "Chef", "Location" };

            var headerRow = sheet.CreateRow(0);

            // create header
            for (int i = 0; i < properties.Length; i++)
            {
                var cell = headerRow.CreateCell(i);
                cell.SetCellValue(headers[i]);
            }

            // fill content 
            for (int i = 0; i < items.Count; i++)
            {
                var rowIndex = i + 1;
                var row = sheet.CreateRow(rowIndex);

                for (int j = 0; j < properties.Length; j++)
                {
                    var cell = row.CreateCell(j);
                    var o = items[i];
                    cell.SetCellValue(o.GetType().GetProperty(properties[j]).GetValue(o, null).ToString());
                }
            }

            using (var stream = new MemoryStream())
            {
                workbook.Write(stream);
                stream.Close();
                return File(stream.ToArray(), "application/vnd.ms-excel", "lunches.xls");
            }
        }
    }
}