using System;
using System.Linq;
using System.Linq.Expressions;
using System.Threading.Tasks;
using Core.Model;
using Data;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Infrastructure;
using Service.Infra.Cache;

namespace Service
{
    public class CrudService
    {
        private readonly ProDbContext efCx;

        private readonly ICacheManager cache;

        public CrudService(ProDbContext efCx, ICacheManager cache)
        {
            this.efCx = efCx;
            this.cache = cache;
        }

        public async Task<T> GetAsync<T>(int id) where T : Entity
        {
            return await efCx.FindAsync<T>(id);
        }

        public async Task LoadProp<T,TProperty>(T item, Expression<Func<T, TProperty>> propertyExpression)
            where TProperty : class
        {
            await efCx.Entry(item).Collection(propertyExpression.GetPropertyAccess().Name).LoadAsync();
        }

        public IQueryable<T> IncludeProp<T,TProperty>(IQueryable<T> items, Expression<Func<T, TProperty>> propertyExpression)
            where T : Entity 
            where TProperty : class
        {
            return items.Include(propertyExpression);
        }

        public async Task Delete<T>(int id) where T : DelEntity
        {
            var entity = await GetAsync<T>(id);

            entity.IsDeleted = true;

            await efCx.SaveChangesAsync();
        }

        public async Task<T[]> GetAll<T>() where T : Entity
        {
            return await efCx.Set<T>().ToArrayAsync();
        }

        public async Task<T[]> GetAll<T>(bool showDeleted) where T : DelEntity
        {
            if (showDeleted)
            {
                return await efCx.Set<T>().ToArrayAsync();
            }

            return await efCx.Set<T>().Where(o => !o.IsDeleted).ToArrayAsync();
        }

        public IQueryable<T> GetQuery<T>(bool showDeleted = false) where T : DelEntity
        {
            if (showDeleted)
            {
                return efCx.Set<T>();
            }

            return efCx.Set<T>().Where(o => !o.IsDeleted);
        }

        public IQueryable<T> GetEntQuery<T>() where T : Entity
        {
            return efCx.Set<T>();
        }

        public async Task<T[]> EvalQueryAsync<T>(IQueryable<T> query) where T : Entity
        {
            return await query.ToArrayAsync();
        }

        public async Task<int> CountAsync<T>(IQueryable<T> query)
        {
            return await query.CountAsync();
        }

        public async Task<T> AddAsync<T>(T entity) where T : Entity
        {
            efCx.Add(entity);
            
            cache.ChangeAction<T>(ChangeType.Create);

            await efCx.SaveChangesAsync();

            return entity;
        }

        public async Task Update<T>(T entity) where T: Entity
        {
            cache.ChangeAction<T>(ChangeType.Edit);
            await efCx.SaveChangesAsync();
        }

        public async Task DeleteAsync<T>(int id) where T : DelEntity
        {
            var ent = await GetAsync<T>(id);
            ent.IsDeleted = true;

            cache.ChangeAction<T>(ChangeType.Delete);
            await efCx.SaveChangesAsync();
        }

        public async Task<T> RestoreAsync<T>(int id) where T: DelEntity
        {
            var ent = await GetAsync<T>(id);
            ent.IsDeleted = false;

            cache.ChangeAction<T>(ChangeType.Edit);
            await efCx.SaveChangesAsync();
            return ent;
        }
    }
}