using Microsoft.AspNetCore.Mvc;
using WebUI.Models;
using Core.Model;
using Microsoft.Extensions.Localization;
using Omu.AwesomeMvc;
using Service;
using Service.Infra.Cache;
using WebUI.Utils;
using WebUI.ViewModels.Display;
using Microsoft.EntityFrameworkCore;

namespace WebUI.Controllers.Awesome
{
    public class DataController : Controller
    {
        private readonly CrudService crudSvc;

        private readonly ICacheManager cache;

        private readonly IStringLocalizer<Mui> localizer;

        public DataController(CrudService crudSvc, ICacheManager cache, IStringLocalizer<Mui> localizer)
        {
            this.crudSvc = crudSvc;
            this.cache = cache;
            this.localizer = localizer;
        }

        public async Task<IActionResult> GetCountries()
        {
            var items =
                await cache.GetAsync("allCountries",
                    async () => await crudSvc.GetAll<Country>(User.IsAdmin()), 
                    typeof(Country));

            return Json(items.Select(o => new KeyContent(o.Id, o.Name)));
        }

        public async Task<IActionResult> GetRoles()
        {
            var items = await crudSvc.GetAll<Role>();

            return Json(items.Select(o => new KeyContent(o.Id, o.Name)));
        }

        public async Task<IActionResult> GetAllChefs()
        {
            var chefs = await cache.GetAsync(CacheKeyGen.New("allChefs", User.IsAdmin()), 
                async () => await crudSvc.GetAll<Chef>(User.IsAdmin()), typeof(Chef));

            var items = new List<KeyContent> { new KeyContent("", localizer["not selected"]) };

            items.AddRange(chefs.Select(o => new KeyContent(o.Id, o.FirstName + " " + o.LastName)));

            return Json(items);
        }

        public async Task<IActionResult> GetMealsInit(int[] v)
        {
            var query = crudSvc.GetQuery<Meal>().Where(o => v.Contains(o.Id))
                .Union(crudSvc.GetQuery<Meal>().OrderByDescending(o => o.Id).Take(10));

            var items = await crudSvc.EvalQueryAsync(query);
            var picUrl = Url.Content(Globals.MealsUrl) + "m";
            return Json(items.Select(o => new MealDisplay(o.Id, o.Name, picUrl + (o.Picture?? "0.jpg"))));
        }

        // used by meals multiselect for remote search
        public async Task<IActionResult> SearchMeals(string term = "")
        {
            var query = crudSvc.GetQuery<Meal>()
                .Where(o => o.Name.Contains(term))
                .Take(10);

            var items = await crudSvc.EvalQueryAsync(query);
            var picUrl = Url.Content(Globals.MealsUrl) + "m";
            return Json(items.Select(o => new MealDisplay(o.Id, o.Name, picUrl + (o.Picture?? "0.jpg"))));
        }

        // used by the country lookup from create dinner popup
        public async Task<ActionResult> GetCountryItem(int? v)
        {
            Check.NotNull(v, nameof(v));

            var country = await crudSvc.GetAsync<Country>(v.Value);

            return Json(new KeyContent(country.Id, country.Name));
        }

        public ActionResult GetMenuNodes()
        {
            var menuNodes =
                MySiteMap.GetAll().Select(
                    o =>
                    new
                    {
                        o.Id,
                        o.Name,
                        o.Keywords,
                        ParentId = o.Parent?.Id,
                        Url = o.Action != null ? Url.Action(o.Action, o.Controller) + o.Anchor : null,
                        o.Action,
                        o.Controller,
                        o.Collapsed,
                        o.NoMenu,
                        o.Anchor
                    });

            return Json(menuNodes);
        }
    }
}