using Core.Model;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Omu.Awem.Utils;
using Omu.AwesomeMvc;
using Service;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using WebUI.Utils;
using WebUI.Utils.Attributes;
using WebUI.Utils.Awe;
using WebUI.ViewModels.Input;

namespace WebUI.Controllers
{
    public class ChefController : Controller
    {
        private readonly CrudService crudSvc;

        private readonly ChefService chefSvc;

        public ChefController(CrudService crudSvc, ChefService chefSvc)
        {
            this.crudSvc = crudSvc;
            this.chefSvc = chefSvc;
        }

        public IActionResult Index()
        {
            return View();
        }

        private object MapToGridModel(Chef o)
        {
            return new
            {
                o.Id,
                o.FirstName,
                o.LastName,
                CountryName = o.Country.Name,

                // gives value to the Country dropdown, when inline editing row
                CountryId = o.Country.Id,
                o.IsDeleted
            };
        }

        public class ChefPrm : FilterPrm
        {
            public string FirstName { get; set; }

            public string LastName { get; set; }

            public int? Country { get; set; }
        }

        public async Task<IActionResult> GridGetItems(GridParams g, ChefPrm filter)
        {
            var query = chefSvc.GetFullQuery(User.IsAdmin());
            
            var filterBuilder = new FilterBuilder<Chef>
            {
                Filter = filter,
                Query = query
            };            

            filterBuilder
                .StringFilter(nameof(ChefPrm.FirstName))
                .StringFilter(nameof(ChefPrm.LastName))
                .Add(nameof(ChefPrm.Country),
                new FilterRule<Chef>
                {
                    Query = q => q.Country.Id == filter.Country,
                    GetData = async q => (await q.Select(o => o.Country)
                        .Distinct()
                        .ToArrayAsync())
                        .Select(o => new KeyContent(o.Id, o.Name))                        
                });

            query = await filterBuilder.ApplyAsync();

            var gmb = new GridModelBuilder<Chef>(query, g)
            {
                KeyProp = o => o.Id,
                Map = MapToGridModel,                
                Tag = filterBuilder.GetGridData(),                
            };

            return Json(await gmb.EFBuildAsync());
        }        

        [HttpPost]
        public async Task<ActionResult> Create(ChefInput input)
        {
            if (ModelState.IsValid)
            {

                var ent = new Chef
                {
                    FirstName = input.FirstName,
                    LastName = input.LastName,
                    Country = await crudSvc.GetAsync<Country>(input.CountryId.Value),
                };

                await crudSvc.AddAsync(ent);

                return Json(new { Item = MapToGridModel(ent) });
            }

            return Json(ModelState.GetErrorsInline());
        }

        [HttpPost]
        public async Task<IActionResult> Edit(ChefInput input)
        {
            if (!ModelState.IsValid) return Json(ModelState.GetErrorsInline());
            

            var ent = await crudSvc.GetAsync<Chef>(input.Id.Value);
            ent.FirstName = input.FirstName;
            ent.LastName = input.LastName;
            ent.Country = await crudSvc.GetAsync<Country>(input.CountryId.Value);

            await crudSvc.Update(ent);

            return Json(new { });
        }

        public async Task<IActionResult> Delete(int id)
        {
            var ent = await crudSvc.GetAsync<Chef>(id);

            return PartialView(new DeleteConfirmInput
            {
                Id = id,
                Type = "chef",
                Name = ent.FirstName + " " + ent.LastName
            });
        }

        [HttpPost]
        public async Task<IActionResult> Delete(DeleteConfirmInput input)
        {
            await crudSvc.DeleteAsync<Chef>(input.Id);

            // the delete PopupForm's success function aweUtils.itemDeleted expects an obj with "Id" property
            return Json(new { input.Id });
        }

        [Admin]
        [HttpPost]
        public async Task<IActionResult> Restore(int id)
        {
            var ent = await crudSvc.RestoreAsync<Chef>(id);

            // obj.id expected in proUtils.restore
            return Json(new { ent.Id });
        }
    }
}