using Core.Model;
using Microsoft.AspNetCore.Mvc;
using Omu.AwesomeMvc;
using Service;
using System.Threading.Tasks;
using WebUI.Utils;
using WebUI.Utils.Attributes;
using WebUI.Utils.Awe;
using WebUI.ViewModels.Input;

namespace WebUI.Controllers
{
    public class CountryController : Controller
    {
        private readonly CrudService crudSvc;

        public CountryController(CrudService crudSvc)
        {
            this.crudSvc = crudSvc;
        }

        public ActionResult LookupGrid()
        {
            return PartialView();
        }

        public IActionResult Index()
        {
            return View();
        }

        private object MapToGridModel(Country o)
        {
            return new
            {
                o.Id, 
                o.Name,

                // used by proUtils.js delRes to determine if it should render delete or restore button
                o.IsDeleted
            };
        }

        public async Task<IActionResult> GridGetItems(GridParams g, string search)
        {
            var query = crudSvc.GetQuery<Country>(User.IsAdmin());
            query = query.Filter(search, o => o.Name);
            
            var gmb = new GridModelBuilder<Country>(query, g)
            {
                KeyProp = o => o.Id,
                Map = MapToGridModel
            };

            return Json(await gmb.EFBuildAsync());
        }

        public IActionResult Create()
        {
            // make sure to use "return PartialView" for PopupForm/Popup views 
            // this will ignore _ViewStart.cshtml so that you don't use the _Layout.cshtml and reload all the scripts
            return PartialView();
        }

        [HttpPost]
        public async Task<IActionResult> Create(CountryInput input)
        {
            if (!ModelState.IsValid) return PartialView(input);
            var ent = await crudSvc.AddAsync(new Country
            {
                Name = input.Name
            });

            // the create PopupForm's success function aweUtils.itemCreated expects the grid row model obj, to render and append the new row
            return Json(MapToGridModel(ent));
        }

        public async Task<IActionResult> Edit(int id)
        {
            Check.IdHasVal(id);

            var ent = await crudSvc.GetAsync<Country>(id);

            var input = new CountryInput
            {
                Id = id, 
                Name = ent.Name
            };

            return PartialView("Create", input);
        }

        [HttpPost]
        public async Task<IActionResult> Edit(CountryInput input)
        {
            if (!ModelState.IsValid) return PartialView("Create", input);
            

            var ent = await crudSvc.GetAsync<Country>(input.Id.Value);

            ent.Name = input.Name;

            await crudSvc.Update(ent);

            // the edit PopupForm's success function aweUtils.itemEdited expects an obj with "Id" property
            return Json(new { ent.Id });
        }

        public async Task<IActionResult> Delete(int id)
        {
            Check.IdHasVal(id);

            var ent = await crudSvc.GetAsync<Country>(id);

            return PartialView(new DeleteConfirmInput
            {
                Id = id,
                Type = "country",
                Name = ent.Name
            });
        }

        [HttpPost]
        public async Task<IActionResult> Delete(DeleteConfirmInput input)
        {
            await crudSvc.DeleteAsync<Country>(input.Id);

            // the delete PopupForm's success function aweUtils.itemDeleted expects an obj with "Id" property
            return Json(new { input.Id });
        }

        [Admin]
        [HttpPost]
        public async Task<IActionResult> Restore(int id)
        {
            Check.IdHasVal(id);

            var ent = await crudSvc.RestoreAsync<Country>(id);
            
            // obj.id expected in proUtils.restore
            return Json(new { ent.Id });
        }
    }
}