using Core.Model;
using Microsoft.AspNetCore.Mvc;
using Omu.AwesomeMvc;
using Omu.AwesomeMvc.DynamicQuery;
using Service;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Threading.Tasks;
using WebUI.Utils;
using WebUI.Utils.Attributes;
using WebUI.Utils.Awe;
using WebUI.ViewModels.Input;

namespace WebUI.Controllers
{
    public class DinnerController : Controller
    {
        private readonly CrudService crudSvc;

        private readonly DinnerService dinnerSvc;

        public DinnerController(CrudService crudSvc, DinnerService dinnerSvc)
        {
            this.crudSvc = crudSvc;
            this.dinnerSvc = dinnerSvc;
        }

        public ActionResult Index()
        {
            return View();
        }

        private object MapToGridModel(Dinner o)
        {
            return new
            {
                o.Id,
                o.IsDeleted,
                o.Name,
                CountryName = o.Country.Name,
                ChefName = o.Chef.FirstName + " " + o.Chef.LastName,
                o.Address,
                MealsCount = o.DinnerMeals.Count,
                MealsStr = string.Join(", ", o.DinnerMeals.Select(dm => dm.Meal.Name)),
                MealsPics = o.DinnerMeals.Select(dm => dm.Meal.Picture),
                Start =  o.Start.ToShortDateString() + " " + o.Start.ToShortTimeString(),
                End = o.End.ToShortTimeString(),
            };
        }

        public async Task<IActionResult> GridGetItems(GridParams g, int? chef, string search)
        {
            var query = dinnerSvc.GetFullQuery(User.IsAdmin());
            query = query.Filter(search, o => o.Name);  

            if (chef.HasValue)
            {
                query = query.Where(o => o.ChefId == chef.Value);
            }
            
            var gmb = new GridModelBuilder<Dinner>(query, g)
            {
                KeyProp = o => o.Id,
                Map = MapToGridModel,
                
                // used on /Dinners page only, when you sort by "Meals" column
                // set this so when ordering by MealsCount to do .OrderBy(o => o.DinnerMeals.Count()) instead (for EF to work, Dinner.MealsCount doesn't exist in the Db table)
                OrderByFunc = (lunches, rules) =>
                {
                    lunches = Dlinq.OrderBy(lunches, rules, new Dictionary<string, LambdaExpression>
                    {
                        { "MealsCount", ExprHelper.GetCountLambda<Dinner>(o => o.DinnerMeals) }
                    });

                    return lunches;
                }
            };            

            return Json(await gmb.EFBuildAsync());
        }

        public IActionResult Create()
        {
            return PartialView();
        }

        [HttpPost]
        public async Task<IActionResult> Create(DinnerInput input)
        {
            if (!ModelState.IsValid) return PartialView(input);


            var ent = new Dinner();

            ent.Name = input.Name;
            ent.Address = input.Address;
            ent.ChefId = input.ChefId.Value;
            ent.CountryId = input.CountryId.Value;
            ent.Start = input.Start.Value.Date + input.Time.Value.TimeOfDay;
            ent.End = ent.Start.AddMinutes(input.Duration);
            ent.DinnerMeals = input.Meals.Select(o => new DinnerMeal { Dinner = ent, MealId = o }).ToList();
            
            await crudSvc.AddAsync(ent);

            ent = await dinnerSvc.GetFullAsync(ent.Id);

            // the create PopupForm's success function aweUtils.itemCreated expects the grid row model obj, to render and append the new row
            return Json(MapToGridModel(ent));
        }

        public async Task<IActionResult> Edit(int id)
        {
            Check.IdHasVal(id);

            var ent = await dinnerSvc.GetFullAsync(id);

            var input = new DinnerInput
            {
                Id = ent.Id,
                Name = ent.Name,
                Address = ent.Address,
                ChefId = ent.ChefId,
                CountryId = ent.CountryId,
                Meals = ent.DinnerMeals.Select(o => o.MealId).ToArray(),
                Start = ent.Start.Date,
                Time = ent.Start,
                Duration = (int) ent.End.Subtract(ent.Start).TotalMinutes
            };

            return PartialView("Create", input);
        }

        [HttpPost]
        public async Task<IActionResult> Edit(DinnerInput input)
        {
            if (!ModelState.IsValid) return PartialView("Create", input);


            var ent = await dinnerSvc.GetFullAsync(input.Id.Value);
            ent.Name = input.Name;
            ent.Address = input.Address;
            ent.ChefId = input.ChefId.Value;
            ent.CountryId = input.CountryId.Value;
            ent.Start = input.Start.Value.Date + input.Time.Value.TimeOfDay;
            ent.End = ent.Start.AddMinutes(input.Duration);
            ent.DinnerMeals = input.Meals.Select(o => new DinnerMeal { Dinner = ent, MealId = o }).ToList();

            await crudSvc.Update(ent);

            return Json(new { ent.Id });
        }

        public async Task<IActionResult> Delete(int id)
        {
            Check.IdHasVal(id);

            var ent = await crudSvc.GetAsync<Dinner>(id);

            return PartialView(new DeleteConfirmInput
            {
                Id = id,
                Type = "dinner",
                Name = ent.Name
            });
        }

        [HttpPost]
        public async Task<IActionResult> Delete(DeleteConfirmInput input)
        {
            await crudSvc.DeleteAsync<Dinner>(input.Id);

            // the delete PopupForm's success function aweUtils.itemDeleted expects an obj with "Id" property
            return Json(new { input.Id });
        }

        [Admin]
        [HttpPost]
        public async Task<IActionResult> Restore(int id)
        {
            Check.IdHasVal(id);
            var ent = await crudSvc.RestoreAsync<Dinner>(id);
            
            // obj.id expected in proUtils.restore
            return Json(new { ent.Id });
        }
    }
}