using Core.Model;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Omu.Awem.Utils;
using Omu.AwesomeMvc;
using Service;
using Service.Infra.Cache;
using System;
using System.IO;
using System.Threading.Tasks;
using WebUI.Utils;
using WebUI.Utils.Attributes;
using WebUI.Utils.Awe;
using WebUI.ViewModels.Input;

namespace WebUI.Controllers
{
    public class MealController : Controller
    {
        private readonly CrudService crudSvc;

        private readonly MealService mealSvc;

        public MealController(CrudService crudSvc, MealService mealSvc)
        {
            this.crudSvc = crudSvc;
            this.mealSvc = mealSvc;
        }

        public IActionResult Index()
        {
            return View();
        }

        private object MapToGridModel(Meal o)
        {
            return new
            {
                o.Id,
                o.Name,
                o.Comments,
                Picture = string.IsNullOrEmpty(o.Picture) ? "0.jpg" : o.Picture,
                o.IsDeleted
            };
        }
        
        public async Task<IActionResult> GridGetData(GridParams g, string search)
        {            
            var isAdmin = User.IsAdmin();

            var query = crudSvc.GetQuery<Meal>(isAdmin);
            query = query.Filter(search, o => o.Name);            
            
            var gmb = new GridModelBuilder<Meal>(query, g)
            {
                KeyProp = o => o.Id,                
                Map = MapToGridModel
            };

            return Json(await gmb.EFBuildAsync());
        }

        public async Task<object> GetMealItem(int key)
        {
            var item = await crudSvc.GetAsync<Meal>(key);
            return MapToGridModel(item);
        }

        [HttpPost]
        public async Task<IActionResult> Edit(MealInput input)
        {
            if (!ModelState.IsValid) return Json(ModelState.GetErrorsInline());

            var edit = input.Id.HasValue;

            var ent = edit ? await crudSvc.GetAsync<Meal>(input.Id.Value) : new Meal();
            ent.Name = input.Name;
            ent.Comments = input.Comments;

            // input.Picture null if user didn't upload a picture 
            if (!string.IsNullOrWhiteSpace(input.Picture))
            {
                await mealSvc.SetPicture(Globals.PicturesPath, input.Picture, ent.Picture);
                ent.Picture = input.Picture;
            }

            if (edit)
            {
                await crudSvc.Update(ent);
            }
            else
            {
                await crudSvc.AddAsync(ent);
            }

            return Json(new { Item = MapToGridModel(ent) });
        }

        public async Task<IActionResult> Delete(int id)
        {
            var ent = await crudSvc.GetAsync<Meal>(id);

            return PartialView(new DeleteConfirmInput
            {
                Id = id,
                Type = "meal",
                Name = ent.Name
            });
        }

        [HttpPost]
        public async Task<IActionResult> Delete(DeleteConfirmInput input)
        {
            await crudSvc.DeleteAsync<Meal>(input.Id);

            // the delete PopupForm's success function aweUtils.itemDeleted expects an obj with "Id" property
            return Json(new { input.Id });
        }

        [Admin]
        [HttpPost]
        public async Task<IActionResult> Restore(int id)
        {
            var ent = await crudSvc.RestoreAsync<Meal>(id);

            // obj.id expected in proUtils.restore  
            return Json(new { ent.Id });
        }

        public IActionResult ChangePicture(string id)
        {
            ViewData["parentId"] = id; // id of the hidden input that stores the picture name

            return PartialView();
        }

        [HttpPost]
        public IActionResult UploadPicture(IFormFile file)
        {
            if (file.Length <= 0 && file.Length < 1024 * 100) throw new AwesomeException("can't upload picture, please try again");
            

            var fileName = Guid.NewGuid() + ".jpg";

            var filePath = Path.Combine(Globals.PicturesPath, MealService.TempPath, fileName);
            using (var stream = new FileStream(filePath, FileMode.Create))
            {
                file.CopyTo(stream);
            }

            return Json(new { name = fileName });
        }
    }
}