using System;
using System.Linq;
using System.Threading.Tasks;
using Core.Model;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Omu.Awem.Utils;
using Omu.AwesomeMvc;
using Service;
using WebUI.Utils.Attributes;
using WebUI.ViewModels.Input;

namespace WebUI.Controllers
{
    [Admin]
    public class UserController : Controller
    {
        private readonly CrudService crudSvc;

        private readonly UserService userSvc;

        public UserController(CrudService crudSvc, UserService userSvc)
        {
            this.crudSvc = crudSvc;
            this.userSvc = userSvc;
        }

        public IActionResult Index()
        {
            return View();
        }

        private object MapToGridModel(User o)
        {
            return new
            {
                o.Id,
                o.Login,
                RolesNames = string.Join(", ", o.UserRoles.Select(ur => ur.Role.Name)),
                o.IsDeleted,

                // gives value to the Roles multiselect, when inline editing row
                Roles = o.UserRoles.Select(ur => ur.RoleId)
            };
        }

        public async Task<IActionResult> GridGetItems(GridParams g, string search)
        {
            search ??= string.Empty;
            var query = userSvc.GetUsersWithRoles()
                .Where(o => o.Login.Contains(search));

            var gmb = new GridModelBuilder<User>(query, g)
            {
                KeyProp = o => o.Id,
                Map = MapToGridModel,
                
                // used when calling grid api.update(id) ( after edit/restore to update the row )
                GetItemAsync = () => userSvc.GetUserWithRoleAsync(Convert.ToInt32(g.Key)),
                
                ToArrayAsync = qitems => qitems.ToArrayAsync()
            };

            return Json(await gmb.BuildAsync());
        }

        public IActionResult ChangePassword()
        {
            return PartialView();
        }

        [HttpPost]
        public async Task<IActionResult> ChangePassword(ChangePasswordInput input)
        {
            if (!ModelState.IsValid) return PartialView(input);

            await userSvc.ChangePassword(input.Id.Value, input.Password);

            // the edit PopupForm's success function aweUtils.itemEdited expects an obj with "Id" property
            return Json(new { Id = input.Id.Value });
        }

        public IActionResult Create()
        {
            return PartialView();
        }

        [HttpPost]
        public async Task<IActionResult> Create(UserCreateInput input)
        {
            if (input.Password != input.ConfirmPassword)
            {
                ModelState.AddModelError("ConfirmPassword", "Passwords don't match");
            }

            if (!ModelState.IsValid) return PartialView(input);

            var ent = new User
            {
                Login = input.Login,
                Password = input.Password
            };

            var currentRoles = input.Roles.Select(o => new UserRole { User = ent, RoleId = o });
            ent.UserRoles = currentRoles.ToList();

            await userSvc.Create(ent);

            var user = await userSvc.GetUserWithRoleAsync(ent.Id);

            return Json(MapToGridModel(user));
        }

        [HttpPost]
        public async Task<IActionResult> Edit(UserEditInput input)
        {
            if (!ModelState.IsValid) return Json(ModelState.GetErrorsInline());

            await userSvc.EditRoles(input.Id.Value, input.Roles);

            // inline edit save success
            return Json(new { });
        }

        public async Task<IActionResult> Delete(int id)
        {
            var ent = await crudSvc.GetAsync<User>(id);

            return PartialView(new DeleteConfirmInput
            {
                Id = id,
                Type = "user",
                Name = ent.Login
            });
        }

        [HttpPost]
        public async Task<IActionResult> Delete(DeleteConfirmInput input)
        {
            await crudSvc.DeleteAsync<User>(input.Id);

            // the delete PopupForm's success function aweUtils.itemDeleted expects an obj with "Id" property
            return Json(new { input.Id });
        }

        [HttpPost]
        public async Task<IActionResult> Restore(int id)
        {
            var ent = await crudSvc.RestoreAsync<User>(id);
            
            // obj.id expected in proUtils.restore
            return Json(new { ent.Id });
        }
    }
}