using Microsoft.AspNetCore.Mvc;
using Omu.StoreAwesome.WebUI.Core.Model;
using Omu.StoreAwesome.WebUI.Utils;
using Omu.AwesomeMvc;
using Omu.StoreAwesome.WebUI.Utils.Attributes;
using SixLabors.ImageSharp;
using SixLabors.ImageSharp.Processing;
using SixLabors.ImageSharp.Formats.Jpeg;
using Omu.StoreAwesome.WebUI.ViewModels.Dto;
using Omu.StoreAwesome.WebUI.Models;
using Omu.StoreAwesome.WebUI.Core.Service;

namespace Omu.StoreAwesome.WebUI.Controllers
{
    [Admin]
    public class AdminProductsController : Controller
    {        
        private readonly ProductService prodSrv;

        public AdminProductsController(ProductService prodSrv)
        {     
            this.prodSrv = prodSrv;
        }

        public IActionResult Index()
        {
            return View();
        }

        private object MapToGridModel(Product p)
        {
            return new
            {
                p.Id,
                p.Name,
                Price = UiUtils.FormatPrice(p.Price),
                Picture = UiUtils.ProdFirstThumbPartPath(p),
                IsActive = p.IsActive ? "yes" : string.Empty,
                p.Priority,
            };
        }

        public async Task<IActionResult> GridGetData(GridParams g, string search)
        {
            var query = prodSrv.GetProducts();

            query = query.FilterProductBySearch(search);

            query = query.OrderByDescending(o => o.Priority).ThenByDescending(o => o.Id);

            var model = new GridModelBuilder<Product>(query, g)
            {
                KeyProp = o => o.Id, // needed for api select, update, tree, nesting, EF                
                Map = MapToGridModel,
                DefaultKeySort = Sort.None
            };

            return Json(await model.EFBuildAsync());
        }

        public IActionResult Add()
        {
            return View("Edit", new ProductInput { Priority = 10 });
        }

        public async Task<IActionResult> Edit(int id)
        {
            var input = await getProductInput(id);

            return View(input);
        }

        [HttpPost]
        public async Task<IActionResult> Save(ProductInput input)
        {
            if (!ModelState.IsValid)
            {
                return View("Edit", input);
            }

            var prod = await prodSrv.Save(input);

            TempData[Const.Success] = true;

            return RedirectToAction("Edit", new { id = prod.Id });
        }

        private async Task<ProductInput> getProductInput(int id)
        {
            var prod = await prodSrv.FindAsync(id);

            var input = new ProductInput
            {
                Id = prod.Id,
                Name = prod.Name,
                Price = prod.Price,
                Description = prod.Description,
                IsActive = prod.IsActive,
                Priority = prod.Priority,
            };

            return input;
        }

        public async Task<IActionResult> RichDescription(int id)
        {
            var input = await getProductInput(id);

            return View(input);
        }

        public async Task<IActionResult> GetRichDescModel(int id)
        {
            var prod = await prodSrv.FindAsync(id);

            return Json(prod.RichDescription);
        }

        public async Task<IActionResult> SaveRichDescModel(int id, string model)
        {
            await prodSrv.UpdateAsync(id, prod =>
            {
                prod.RichDescription = model;
            });

            return Json(new { });
        }

        #region pictures
        public async Task<IActionResult> SelectPicture(int productId)
        {
            var prod = await prodSrv.FindAsync(productId);

            return PartialView(prod);
        }

        public async Task<IActionResult> Pictures(int id)
        {
            var input = await getProductInput(id);

            return View(input);
        }

        private string getProductPicDir(int productId)
        {
            var prodPicDir = Path.Combine(Globals.ProductPicsPath, productId.ToString());

            return prodPicDir;
        }

        private static string getThumbFileName(string fileName)
        {
            return $"t_{fileName}";
        }

        [HttpPost]
        public async Task<IActionResult> SavePictures(PicInput[] inputs, int productId)
        {
            // new pictures have File set and Name null
            var newFiles = inputs.Where(p => p.File != null && p.Name == null);

            if (newFiles.Any(pic => pic.File.Length < 1024 * 3))
            {
                throw new AwesomeException("can't upload picture, please try again");
            }

            var prod = await prodSrv.FindAsync(productId);

            var prodPicDir = getProductPicDir(productId);

            Directory.CreateDirectory(prodPicDir);
            var filesToSave = new List<string>();

            foreach (var pic in newFiles)
            {
                prod.Version++;
                var newFileName = $"{prod.Version}.jpg";
                var thumbFileName = getThumbFileName(newFileName);
                pic.Name = newFileName;

                filesToSave.Add(newFileName);

                var filePath = Path.Combine(prodPicDir, newFileName);
                var filePathThumb = Path.Combine(prodPicDir, thumbFileName);

                using var fstream = pic.File.OpenReadStream();

                var img = Image.Load(fstream);

                // needed for transparent images, webp https://github.com/SixLabors/ImageSharp/discussions/2659
                img.Mutate(x => { x.BackgroundColor(Color.White); });

                if (pic.CropHeight is not null)
                {
                    img.Mutate(op =>
                    op.Crop(new Rectangle(pic.CropX.Value, pic.CropY.Value, pic.CropWidth.Value, pic.CropHeight.Value)));
                }

                var thumb = img.Clone(limitHW(img, 300, 300));

                await thumb.SaveAsync(filePathThumb, new JpegEncoder { Quality = 50 });

                limitWidthAndHeight(img, 1000, 1000);

                await img.SaveAsync(filePath, new JpegEncoder { Quality = 50 });
            }

            // save version
            await prodSrv.UpdateAsync(prod.Id, p => { p.Version = prod.Version; });

            var errors = new List<string>();

            var exFiles = Directory.GetFiles(prodPicDir).Select(Path.GetFileName).ToList();

            var dbpics = inputs.Select(pic => new PicDbDto { Name = pic.Name }).ToList();

            await prodSrv.UpdateAsync(prod.Id, p =>
            {               
                p.Picture = JsonUtils.Serialize(dbpics);
            });

            Globals.ProductsListVersion = await prodSrv.GetProductsListVersion();

            var unknownFiles = exFiles.Except(
                inputs.Select(p => p.Name)
                      .Union(inputs.Select(p => getThumbFileName(p.Name)))
                );

            if (unknownFiles.Any())
            {
                errors.Add("found unlisted files in the pictures folder: " + JsonUtils.Serialize(unknownFiles));
            }

            return Json(new { errors });
        }

        private Action<IImageProcessingContext> limitHW(Image img, int limWidth, int limHeight)
        {
            return op =>
            {
                if (img.Width > limWidth)
                {
                    var neww = limWidth;
                    var newh = img.Height * neww / img.Width;
                    op.Resize(neww, newh);
                }
                else if (img.Height > limHeight)
                {
                    var newh = limHeight;
                    var neww = img.Width * newh / img.Height;
                    op.Resize(neww, newh);
                }
            };
        }

        private Image limitWidthAndHeight(Image img, int limWidth, int limHeight)
        {
            if (img.Width > limWidth)
            {
                var neww = limWidth;
                var newh = img.Height * neww / img.Width;
                img.Mutate(x => x.Resize(neww, newh));
            }
            else if (img.Height > limHeight)
            {
                var newh = limHeight;
                var neww = img.Width * newh / img.Height;
                img.Mutate(x => x.Resize(neww, newh));
            }

            return img;
        }

        public async Task<IActionResult> GetPictures(int productId)
        {
            var prod = await prodSrv.FindAsync(productId);

            var pictures = JsonUtils.TryDeserialize<PicDbDto[]>(prod.Picture);

            if (pictures == null)
            {
                return Json(new object[] { });
            }

            var res = new List<PicDisplay>();

            foreach (var p in pictures)
            {
                var disp = new PicDisplay();
                disp.Name = p.Name;
                var fpath = Path.Combine(getProductPicDir(productId), p.Name);

                disp.Size = FileUtil.GetSizeFE(fpath);

                res.Add(disp);
            }

            return Json(res);
        }

        public async Task<IActionResult> DeletePicture(int productId, string name)
        {
            var prod = await prodSrv.FindAsync(productId);

            var pictures = JsonUtils.Deserialize<PicDbDto[]>(prod.Picture).ToList();

            var picToDelete = pictures.FirstOrDefault(o => o.Name == name);

            if (picToDelete != null)
            {
                pictures.Remove(picToDelete);
                FileUtil.DeletePicture(productId, picToDelete.Name);
                FileUtil.DeletePicture(productId, getThumbFileName(picToDelete.Name));
            }

            await prodSrv.UpdateAsync(prod.Id, p =>
            {
                p.Picture = JsonUtils.Serialize(pictures);
            });

            return Json(pictures);
        }
        #endregion

        #region models
        public class PicDisplay
        {
            public string Name { get; set; }

            public long Size { get; set; }
        }

        public class PicInput
        {
            // used to get file from client
            public IFormFile File { get; set; }

            /// <summary>
            /// Original uploaded file name 
            /// </summary>
            public string OrigName { get; set; }

            // File name on server
            public string Name { get; set; }

            public int? CropX { get; set; }
            public int? CropY { get; set; }
            public int? CropHeight { get; set; }
            public int? CropWidth { get; set; }
        }
        #endregion
    }
}