using Microsoft.AspNetCore.Mvc;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using Omu.StoreAwesome.WebUI.Core.Model;
using Omu.StoreAwesome.WebUI.Core.Repo;
using Omu.StoreAwesome.WebUI.Core.Service;

namespace Omu.StoreAwesome.WebUI.Controllers
{
    public class CheckoutController : Controller
    {
        private readonly ProductService prodRepo;

        private readonly GenericBaseRepo<Order> orderRepo;

        public CheckoutController(GenericBaseRepo<Order> orderRepo, ProductService prodRepo)
        {
            this.orderRepo = orderRepo;
            this.prodRepo = prodRepo;
        }        

        public IActionResult Index()
        {
            return View();
        }

        [HttpPost]
        public async Task<IActionResult> Index(CartItem[] cartItems, OrderInput orderInput)
        {
            if (orderInput.Email != orderInput.ConfirmEmail)
            {
                ModelState.AddModelError(nameof(OrderInput.Email), "E-mail doesn't match Confirm E-Mail");
            }

            if (cartItems.Length == 0)
            {
                ModelState.AddModelError(nameof(OrderInput.Email), "Your Store Awesome Cart is empty");
            }

            decimal subtotal = 0;

            var orderItems = new List<OrderItem>();
            foreach (var cartItem in cartItems)
            {
                var prod = await prodRepo.GetActiveProduct(cartItem.ItemId);

                if (prod is null)
                {
                    ModelState.AddModelError("Generic", "One of the items in your shopping cart is not currently available, the item has been removed from your cart.");
                }
                else if (prod.Price != cartItem.Price)
                {
                    ModelState.AddModelError("Generic",
                        $"The price for the item {prod.Name} has changed from {cartItem.Price} to {prod.Price}, please check and submit again.");
                }
                else
                {
                    var orderItem = new OrderItem
                    {
                        UnitPrice = cartItem.Price,
                        Quantity = cartItem.Quantity,
                        ProductId = cartItem.ItemId,
                        ProductName = prod.Name
                    };

                    orderItems.Add(orderItem);

                    subtotal += orderItem.UnitPrice * orderItem.Quantity;
                }
            }

            if (!ModelState.IsValid)
            {
                return View(orderInput);
            }

            // save order
            var order = new Order
            {
                FirstName = orderInput.FirstName,
                LastName = orderInput.LastName,
                Email = orderInput.Email,

                StreetAddress = orderInput.StreetAddress,
                City = orderInput.City,
                State = orderInput.State,
                ZipCode = orderInput.ZipCode,

                SubTotal = subtotal,

                OrderItems = orderItems,
                DateCreated = DateTime.UtcNow
            };

            await orderRepo.AddAsync(order);            

            return RedirectToAction("Success", new { id = order.Id });
        }

        public IActionResult Success(int id)
        {
            ViewData["id"] = id;
            return View();
        }

        public class CartItem
        {
            public int ItemId { get; set; }
            public int Quantity { get; set; }
            public decimal Price { get; set; }
        }

        public class OrderInput
        {
            [Required]
            [DisplayName("E-mail")]
            public string Email { get; set; }

            [Required]
            [DisplayName("Confirm e-mail")]
            public string ConfirmEmail { get; set; }

            [Required]
            [DisplayName("First Name")]
            public string FirstName { get; set; }

            [Required]
            [DisplayName("Last Name")]
            public string LastName { get; set; }

            [Required]
            [DisplayName("Street address")]
            public string StreetAddress { get; set; }

            [Required]
            public string City { get; set; }

            [Required]
            public string State { get; set; }

            [Required]
            [DisplayName("ZIP Code")]
            public string ZipCode { get; set; }
        }
    }
}