using ElmahCore;
using ElmahCore.Mvc;
using Microsoft.AspNetCore.Authentication.Cookies;
using Microsoft.AspNetCore.DataProtection;
using Microsoft.EntityFrameworkCore;
using Omu.AwesomeMvc;
using Serilog;
using Omu.StoreAwesome.WebUI.Utils;
using Omu.StoreAwesome.WebUI.Utils.BackgroundWorker;
using Microsoft.Extensions.Options;
using Omu.StoreAwesome.WebUI.Core.Repo;
using Omu.StoreAwesome.WebUI.Core.Service;
using Omu.StoreAwesome.WebUI.Core.Data;
using Omu.StoreAwesome.WebUI.Core.Data.Mongo;

// use MongoDb when true, (MONGODB_URI Env Var)
// or SQL Server when false (connection string in StoreSQL_CS Env Var or appsettings.json)
bool useMongoDb = false;

try
{
    Log.Information("Starting web application");
    main();

}
catch (Exception ex)
{
    Log.Fatal(ex, "Application terminated unexpectedly");
    throw;
}
finally
{
    Log.CloseAndFlush();
}

void main()
{
    var builder = WebApplication.CreateBuilder(args);

    Globals.PicturesPath = Path.Combine(builder.Environment.WebRootPath, "pictures");
    Globals.LogsPath = Path.Combine(builder.Environment.ContentRootPath, Path.Combine("App_Data","logs"));
    Globals.ElmahLogsPath = Path.Combine(builder.Environment.ContentRootPath, Path.Combine("App_Data", "elogs"));

    if (builder.Environment.IsDevelopment())
    {
        setupDevRemoteAccess(builder);
    }    

    readAppConfig(builder);

    setupElmah(builder);

    setupSerilog(builder);    

    // for persistent cookie to work
    builder.Services.AddDataProtection()
        .PersistKeysToFileSystem(new DirectoryInfo("App_Data/PersKey"));

    builder.Services.AddAuthentication(CookieAuthenticationDefaults.AuthenticationScheme)
        .AddCookie(opt => opt.LoginPath = "/Admin/SignIn");       

    // Add services to the container.
    builder.Services.AddControllersWithViews();

    builder.Services.AddHsts(options =>
    {
        options.IncludeSubDomains = false;
        options.MaxAge = TimeSpan.FromDays(1);
    });

    builder.Services.AddHttpsRedirection(options =>
    {
        options.RedirectStatusCode = StatusCodes.Status308PermanentRedirect;
        options.HttpsPort = 443;
    });

    addServices(builder);

    // Add framework services.
    var mvcBuilder = builder.Services.AddControllersWithViews(o =>
    {
        o.ModelMetadataDetailsProviders.Add(new AweMetaProvider());
    })
    .AddJsonOptions(opt =>
    {
        opt.JsonSerializerOptions.PropertyNamingPolicy = null;
    });

    mvcBuilder.AddRazorRuntimeCompilation();

    builder.Services.AddRouting(cfg => {
        cfg.LowercaseUrls = true;
    });

    if (useMongoDb)
    {
        setupMongo(builder);
    }
    {
        setupSqlDatabase(builder);
    }

    var app = builder.Build();
    //app.UseMiniProfiler();

    
    //initDataMigration(app);

    initProductListVersion(app);
    
    // Configure the HTTP request pipeline.
    //var noAutoHttps = app.Configuration["STOREAPP_NOAUTOHTTPS"] == "true"; // set in dockerfile    
    //if (!app.Environment.IsDevelopment() && !noAutoHttps)
    //{       
    //    app.UseHsts();
    //    app.UseHttpsRedirection();
    //}

    app.UseExceptionHandler("/Error");
    app.UseStatusCodePagesWithReExecute("/Error/HttpStatus", "?code={0}");

    app.UseStaticFiles();

    app.UseRouting();

    app.UseAuthorization();

    #region routing
    app.MapControllerRoute(
    "ViewProduct",
    "{title?}/prod/{id?}",
    new { controller = "Product", action = "OpenProduct" });

    app.MapControllerRoute(
   "Search",
   "/s",
   new { controller = "Home", action = "Search" });

    app.MapControllerRoute(
        name: "default",
        pattern: "{controller=Home}/{action=Index}/{id?}");
    #endregion    

    app.UseElmah();
    app.Run();
}

void addServices(WebApplicationBuilder builder)
{
    if (!useMongoDb)
    {
        // using sql server db
        builder.Services.AddScoped<IProdRepo, ProdSqlRepo>();
        builder.Services.AddScoped(typeof(GenericBaseRepo<>), typeof(GenericSqlRepo<>));
    }
    else
    {
        // mongo db context
        #region Mongo Db
        builder.Services.AddSingleton<MongoDbFactory, MongoDbFactory>();
        builder.Services.AddSingleton<DbContextOptions<MongoDbContext>, DbContextOptions<MongoDbContext>>(svp => svp.GetService<MongoDbFactory>().GetOptions());
        builder.Services.AddSingleton<MongoDbContext, MongoDbContext>(svp => svp.GetService<MongoDbFactory>().GetContext());
        builder.Services.AddScoped<MongoDbContext, MongoDbContext>();

        builder.Services.AddScoped<IProdRepo, ProdMongoRepo>();
        builder.Services.AddScoped(typeof(GenericBaseRepo<>), typeof(GenericMongoRepo<>));
        #endregion
    }

    builder.Services.AddScoped<IAppInfoRepo, AppInfoRepo>();
    builder.Services.AddScoped<AccountService, AccountService>();
    builder.Services.AddScoped<ProductService, ProductService>();

    // background worker
    builder.Services.AddHostedService<ConsumeScopedServiceHostedService>();
    builder.Services.AddScoped<IScopedProcessingService, ScopedProcessingService>();
}

void setupSqlDatabase(WebApplicationBuilder builder)
{
    // sql server db    
    var connectionString = Environment.GetEnvironmentVariable("StoreSQL_CS");

    if (string.IsNullOrWhiteSpace(connectionString))
    {
        connectionString = builder.Configuration.GetConnectionString("DefaultConnection");
    }

    builder.Services.AddDbContext<SqlDbContext>(o => o.UseSqlServer(connectionString));
}

void readAppConfig(WebApplicationBuilder builder)
{
    Globals.Config = new ConfigInfo
    {
        CaptchaSecret = builder.Configuration["CaptchaSecret"],
        CaptchaSiteKey = builder.Configuration["CaptchaSiteKey"]
    };
}

void setupDevRemoteAccess(WebApplicationBuilder builder)
{
    if (builder.Environment.IsDevelopment())
    {
        // to access site (running from VS http profile) from a different device on the network
        builder.WebHost
        .UseUrls("http://0.0.0.0:5222;")
        .UseKestrel();
    }
}

void setupSerilog(WebApplicationBuilder builder)
{
    builder.Host.UseSerilog(
            (context, configuration) => configuration.ReadFrom.Configuration(context.Configuration));
}

void setupElmah(WebApplicationBuilder builder)
{
    builder.Services.AddElmah<XmlFileErrorLog>(options =>
    {
        options.Path = "elmahlogs";        
        options.LogPath = Globals.ElmahLogsPath;

        options.OnPermissionCheck = context => context.User.IsInRole("admin");
    });
}

void setupMongo(WebApplicationBuilder builder)
{
    var connectionString = Environment.GetEnvironmentVariable("MONGODB_URI");
    if (connectionString == null)
    {
        Console.WriteLine("You must set your 'MONGODB_URI' environment variable. To learn how to set it, see https://www.mongodb.com/docs/drivers/csharp/current/quick-start/#set-your-connection-string");
        Environment.Exit(0);
    }
    
    builder.Services.Configure<Omu.StoreAwesome.WebUI.Core.Data.Mongo.MongoDatabaseSettings>(
        builder.Configuration.GetSection(nameof(Omu.StoreAwesome.WebUI.Core.Data.Mongo.MongoDatabaseSettings)));

    builder.Services.AddSingleton(sp =>
    {
        var val = sp.GetRequiredService<IOptions<Omu.StoreAwesome.WebUI.Core.Data.Mongo.MongoDatabaseSettings>>().Value;

        val.ConnectionString = connectionString;

        return val;
    });
}

void initProductListVersion(WebApplication app)
{
    using (var scope = app.Services.CreateScope())
    {
        var mcx = scope.ServiceProvider.GetService<IAppInfoRepo>();

        var appInfo = mcx.GetAsync().Result;

        if (appInfo != null)
        {
            Globals.ProductsListVersion = appInfo.ProductsListVersion;
        }
    }
}

void initDataMigration(WebApplication app)
{
    using (var scope = app.Services.CreateScope())
    {
        var sqlCx = scope.ServiceProvider.GetService<SqlDbContext>();

        #region rem MongoId
        //var mongoFactory = scope.ServiceProvider.GetService<MongoDbFactory>();

        //var coll = mongoFactory.GetCollection<Product>();

        //// Cr�ez un filtre pour s�lectionner tous les documents.
        //var filter = new BsonDocument();

        //// Cr�ez une d�finition de mise � jour pour supprimer le champ.
        //var update = Builders<Product>.Update.Unset("MongoId");

        //// Ex�cutez la mise � jour.
        //coll.UpdateMany(filter, update);
        #endregion

        #region move data from ef to mongo
        //var mongoCx = scope.ServiceProvider.GetService<MongoDbContext>();        

        //var prods = sqlCx.Products.ToList();

        //foreach (var product in prods)
        //{
        //    var mongoProd = mongoCx.Products.Where(p => p.Id != null)
        //        .FirstOrDefault(p => (int)p.Id == (int)product.Id);

        //    if (mongoProd != null) continue;

        //    mongoCx.Products.Add(product);
        //}

        //mongoCx.SaveChanges(); 
        #endregion
    }
}