using ElmahCore;
using Serilog;

namespace Omu.StoreAwesome.WebUI.Utils.BackgroundWorker
{
    /// <summary>
    /// https://learn.microsoft.com/en-us/aspnet/core/fundamentals/host/hosted-services?view=aspnetcore-7.0&tabs=visual-studio
    /// </summary>
    public class ConsumeScopedServiceHostedService : BackgroundService
    {
        public ConsumeScopedServiceHostedService(IServiceProvider services)
        {
            Services = services;
        }

        public IServiceProvider Services { get; }

        protected override async Task ExecuteAsync(CancellationToken stoppingToken)
        {
            await DoWork(stoppingToken);
        }

        private async Task DoWork(CancellationToken stoppingToken)
        {
            using (var scope = Services.CreateScope())
            {
                var scopedProcessingService =
                    scope.ServiceProvider
                        .GetRequiredService<IScopedProcessingService>();

                await scopedProcessingService.DoWork(stoppingToken);
            }
        }

        public override async Task StopAsync(CancellationToken stoppingToken)
        {
            await base.StopAsync(stoppingToken);
        }
    }

    internal interface IScopedProcessingService
    {
        Task DoWork(CancellationToken stoppingToken);
    }

    internal class ScopedProcessingService : IScopedProcessingService
    {
        public ScopedProcessingService(IServiceProvider services)
        {
            Services = services;
        }

        public IServiceProvider Services { get; }

        public async Task DoWork(CancellationToken stoppingToken)
        {
            while (!stoppingToken.IsCancellationRequested)
            {
                try
                {
                    await Iteration();
                }
                catch (Exception ex)
                {
                    ElmahExtensions.RaiseError(ex);
                    Log.Error(ex, ex.Message);
                }

                // each minute
                await Task.Delay(1 * 60 * 1000, stoppingToken);
            }
        }

        private async Task Iteration()
        {
            try
            {
                //var mcx = Services.GetService<MyDbContext>();

                // run at utc midnight
                if (DateTime.UtcNow.TimeOfDay.TotalMinutes < 5)
                {
                    deleteElmahLogsAfterLimit();
                    deleteLogsAfterLimit();
                }
            }
            catch (Exception ex)
            {
                Log.Error(ex, ex.Message);
            }
        }

        private void deleteElmahLogsAfterLimit()
        {
            var elmahFilesLimit = 5000;
            var dirInfo = new DirectoryInfo(Globals.ElmahLogsPath);
            var efiles = dirInfo.GetFiles();

            if (efiles.Length > elmahFilesLimit)
            {
                var toDelete = efiles.OrderBy(f => f.LastWriteTime).Skip(elmahFilesLimit).ToArray();

                foreach (var file in toDelete)
                {
                    File.Delete(file.FullName);
                }
            }
        }

        private void deleteLogsAfterLimit()
        {
            var elmahFilesLimit = 500;
            var dirInfo = new DirectoryInfo(Globals.LogsPath);
            var efiles = dirInfo.GetFiles();

            if (efiles.Length > elmahFilesLimit)
            {
                var toDelete = efiles.OrderBy(f => f.LastWriteTime).Skip(elmahFilesLimit).ToArray();

                foreach (var file in toDelete)
                {
                    File.Delete(file.FullName);
                }
            }
        }
    }
}