using System.Linq.Expressions;

namespace Omu.StoreAwesome.WebUI.Utils
{
    public static class ExprUtil
    {
        private static Expression<Func<TModel, bool>> MakeContainsToLower<TModel>(
            IEnumerable<ParameterExpression> parameters, // lambda param
            Expression argument, // contains method arguments
            MemberExpression prop)
        {
            var containsMethodInfo = typeof(string).GetMethod("Contains", new[] { typeof(string) });
            var toLowerMethodInfo = typeof(string).GetMethod("ToLower", System.Type.EmptyTypes);

            var mainExpr = Expression.Call(prop, toLowerMethodInfo);
            mainExpr = Expression.Call(mainExpr, containsMethodInfo, argument);

            return Expression.Lambda<Func<TModel, bool>>(mainExpr, parameters);
        }

        public static Expression<Func<TModel, bool>> ContainsWordsToLowerAndExpr<TModel>(
            string[] keywords, 
            Expression<Func<TModel, string>> propExpr, 
            Expression<Func<TModel, bool>> customAndExpr = null)
        {
            Expression<Func<TModel, bool>> res = null;

            foreach (var keyword in keywords)
            {
                var propAcc = propExpr.Body as MemberExpression;
                var contLambda = MakeContainsToLower<TModel>(propExpr.Parameters, Expression.Constant(keyword, typeof(string)), propAcc);

                if (res == null)
                {
                    res = contLambda;
                }
                else
                {
                    res = res.And(contLambda);
                }
            }

            if (customAndExpr != null)
            {
                res = res.And(customAndExpr);
            }

            return res;
        }
    }

    static class Extensions
    {
        public static Expression<Func<T, bool>> Or<T>(
            this Expression<Func<T, bool>> expr1,
            Expression<Func<T, bool>> expr2)
        {
            var parameter = Expression.Parameter(typeof(T));

            var leftVisitor = new ReplaceExpressionVisitor(expr1.Parameters[0], parameter);
            var left = leftVisitor.Visit(expr1.Body);

            var rightVisitor = new ReplaceExpressionVisitor(expr2.Parameters[0], parameter);
            var right = rightVisitor.Visit(expr2.Body);

            return Expression.Lambda<Func<T, bool>>(
                Expression.Or(left, right), parameter);
        }

        public static Expression<Func<T, bool>> And<T>(
           this Expression<Func<T, bool>> expr1,
           Expression<Func<T, bool>> expr2)
        {
            var parameter = Expression.Parameter(typeof(T));

            var leftVisitor = new ReplaceExpressionVisitor(expr1.Parameters[0], parameter);
            var left = leftVisitor.Visit(expr1.Body);

            var rightVisitor = new ReplaceExpressionVisitor(expr2.Parameters[0], parameter);
            var right = rightVisitor.Visit(expr2.Body);

            return Expression.Lambda<Func<T, bool>>(
                Expression.And(left, right), parameter);
        }
    }

    class ReplaceExpressionVisitor : ExpressionVisitor
    {
        private readonly Expression _oldValue;
        private readonly Expression _newValue;

        public ReplaceExpressionVisitor(Expression oldValue, Expression newValue)
        {
            _oldValue = oldValue;
            _newValue = newValue;
        }

        public override Expression Visit(Expression node)
        {
            if (node == _oldValue)
                return _newValue;
            return base.Visit(node);
        }
    }
}