using Microsoft.AspNetCore.Mvc;
using Microsoft.IdentityModel.Tokens;
using Newtonsoft.Json;
using Omu.Awem.Utils;
using Omu.AwesomeMvc;
using Serilog;
using System.Linq.Expressions;
using System.Text;
using Omu.StoreAwesome.WebUI.Core.Model;
using Omu.StoreAwesome.WebUI.ViewModels.Dto;
using static Microsoft.EntityFrameworkCore.DbLoggerCategory;

namespace Omu.StoreAwesome.WebUI.Utils
{
    public static class AweDataExtensions
    {
        /// <summary>
        /// will call Where(o => o.Prop.Contains(term) || o.Prop2.Contains(term) ...)
        /// depending on Settings.StringFilterIgnoreCase will either use Contains (works with EF),
        /// or IndexOf(term, StringComparison.InvariantCultureIgnoreCase)
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="query"></param>
        /// <param name="term"></param>
        /// <param name="props"></param>
        /// <returns></returns>
        public static IQueryable<T> Filter<T>(this IQueryable<T> query, string term, params Expression<Func<T, string>>[] props)
        {
            if (term == null) return query;

            if (Settings.StringFilterIgnoreCase)
            {
                return query.Where(AweExprUtil.IndexOf(term, props));
            }

            return query.Where(AweExprUtil.ContainsExpr(term, props));
        }

        /// <summary>
        /// Filter query property by multiple words using AND ( where o.Prop1.Contains(word[0]) and o.Prop1.Contains(word[1]) and ... )
        /// </summary>
        public static IQueryable<T> FilterByWordsAnd<T>(this IQueryable<T> query, Expression<Func<T, string>> propExpr, string[] keywords)
        {
            if (keywords == null || keywords.Length == 0) return query;

            return query.Where(AweExprUtil.ContainsWordsAndExpr(keywords, propExpr));
        }

        public static IQueryable<Product> FilterProductBySearch(this IQueryable<Product> query, string search)
        {
            if (string.IsNullOrEmpty(search))
            {
                return query;
            }

            var terms = UiUtils.GetKeywords(search);
            Expression<Func<Product, string>> propExpr = o => o.Name;
            query = query.Where(ExprUtil.ContainsWordsToLowerAndExpr(terms, propExpr));
            return query;
        }
    }

    public static class UiUtils
    {
        public static string[] GetKeywords(string term)
        {
            return term.Split(' ').Select(x => x.Trim()).Where(o => !string.IsNullOrWhiteSpace(o)).ToArray();
        }
        public static int GetPageCount(int count, int pageSize)
        {
            return (int)Math.Ceiling(count / (float)pageSize);
        }

        public static IQueryable<T> GetPage<T>(IQueryable<T> query, int page, int pageSize)
        {
            return query.Skip((page - 1) * pageSize).Take(pageSize);
        }

        public static string FormatPrice(decimal price)
        {
            return price.ToString("0.00");
        }

        public static string GetQueryStringKeyword(HttpContext context)
        {
            return context.Request.Query["k"];
        }

        public static string ProdFirstThumbPartPath(IPictureEntity product)
        {
            return ProdFirstPartPath(product, true);
        }

        public static string ProdFirstThumbPartPath(string picturesJson, int id)
        {
            return ProdFirstPartPath(picturesJson, id);
        }

        public static string ProdThumbImgPath(IUrlHelper Url, int productId, string fileName)
        {
            return @Url.Content("~/pictures/Products/" + $"{productId}/t_{fileName}");
        }

        public static string ProdImgPath(IUrlHelper Url, int productId, string fileName)
        {
            return @Url.Content("~/pictures/Products/" + $"{productId}/{fileName}");
        }

        public static string ProdFirstPartPath(IPictureEntity product, bool thumb = false)
        {
            return ProdFirstPartPath(product.Picture, product.Id.Value, thumb);
        }

        public static string ProdFirstPartPath(string picturesJson, int id, bool thumb = false)
        {
            var pics = JsonUtils.TryDeserialize<PicDbDto[]>(picturesJson);

            if (pics.IsNullOrEmpty())
            {
                return "0.jpg";
            }

            var name = pics.First().Name;

            if (thumb)
            {
                name = "t_" + name;
            }

            return $"{id}/{name}";

        }

        public static string PicFormat(IUrlHelper Url)
        {
            return "<img src='" + Url.Content("~/pictures/Products/.(Picture)") + "' alt='.(Name)' />";
        }

        public static string DeleteUplBtn()
        {
            return "<button type=\"button\" class=\"awe-btn\" onclick=\"sitePictures.deleteUplImg('.(name)')\">Delete</button>";
        }

        public static string PopupDetailsOpenBtn(string text, string popupName)
        {
            return $"<button type=\"button\" class=\"awe-btn\" onclick=\"awe.open('{popupName}', {{ params: {{ id: '.(Id)' }} }})\">{text}</button>";
        }

        public static string UrlFriendly(string title)
        {
            if (title == null) return "";

            const int Maxlen = 80;
            var len = title.Length;
            var prevdash = false;
            var sb = new StringBuilder(len);
            char c;

            for (int i = 0; i < len; i++)
            {
                c = title[i];
                if ((c >= 'a' && c <= 'z') || (c >= '0' && c <= '9'))
                {
                    sb.Append(c);
                    prevdash = false;
                }
                else if (c >= 'A' && c <= 'Z')
                {
                    // tricky way to convert to lowercase
                    sb.Append((char)(c | 32));
                    prevdash = false;
                }
                else if (c == ' ' || c == ',' || c == '.' || c == '/' || c == '\\' || c == '-' || c == '_' || c == '=')
                {
                    if (!prevdash && sb.Length > 0)
                    {
                        sb.Append('-');
                        prevdash = true;
                    }
                }
                else if ((int)c >= 128)
                {
                    int prevlen = sb.Length;
                    sb.Append(RemapInternationalCharToAscii(c));
                    if (prevlen != sb.Length) prevdash = false;
                }
                if (i == Maxlen) break;
            }

            if (prevdash)
                return sb.ToString().Substring(0, sb.Length - 1);
            else
                return sb.ToString();
        }

        public static string RemapInternationalCharToAscii(char c)
        {
            string s = c.ToString().ToLowerInvariant();
            if ("àåáâäãåą".Contains(s))
            {
                return "a";
            }
            else if ("èéêëę".Contains(s))
            {
                return "e";
            }
            else if ("ìíîïı".Contains(s))
            {
                return "i";
            }
            else if ("òóôõöøőð".Contains(s))
            {
                return "o";
            }
            else if ("ùúûüŭů".Contains(s))
            {
                return "u";
            }
            else if ("çćčĉ".Contains(s))
            {
                return "c";
            }
            else if ("żźž".Contains(s))
            {
                return "z";
            }
            else if ("śşšŝ".Contains(s))
            {
                return "s";
            }
            else if ("ñń".Contains(s))
            {
                return "n";
            }
            else if ("ýÿ".Contains(s))
            {
                return "y";
            }
            else if ("ğĝ".Contains(s))
            {
                return "g";
            }
            else if (c == 'ř')
            {
                return "r";
            }
            else if (c == 'ł')
            {
                return "l";
            }
            else if (c == 'đ')
            {
                return "d";
            }
            else if (c == 'ß')
            {
                return "ss";
            }
            else if (c == 'Þ')
            {
                return "th";
            }
            else if (c == 'ĥ')
            {
                return "h";
            }
            else if (c == 'ĵ')
            {
                return "j";
            }
            else
            {
                return "";
            }
        }
    }

    public static class JsonUtils
    {
        public static string Serialize(object o)
        {
            return JsonConvert.SerializeObject(o, Formatting.None, new JsonSerializerSettings { StringEscapeHandling = StringEscapeHandling.EscapeHtml });
        }

        public static T Deserialize<T>(string str)
        {
            return JsonConvert.DeserializeObject<T>(str);
        }

        public static T TryDeserialize<T>(string str) where T : class
        {
            try
            {
                return JsonConvert.DeserializeObject<T>(str);
            }
            catch (Exception ex)
            {
                Log.Error(ex, ex.Message);                
                return null;
            }
        }
    }

    public static class FileUtil
    {
        /// <summary>
        /// get file size in bytes, 0 if file not found
        /// </summary>        
        public static long GetSizeFE(string path)
        {
            if (!File.Exists(path)) return 0;

            return new FileInfo(path).Length;
        }

        public static bool FileExists(string path)
        {
            return File.Exists(path);
        }

        public static void Delete(string path)
        {
            if (File.Exists(path))
            {
                File.Delete(path);
            }
        }

        public static void DeletePicture(int productId, string name)
        {
            var path = Path.Combine(Globals.ProductPicsPath, productId.ToString(), name);

            Delete(path);
        }
    }

    public static class Const
    {
        public const string Success = "success";

        public const string CartNoCache = "CartNoCache";

        public const string NoCornerCart = "NoCornerCart";
    }

    public static class Globals
    {
        public static string LogsPath { get; set; }
        
        public static string ElmahLogsPath { get; set; }

        public static string PicturesPath { get; set; }

        public static string ProductPicsPath { get { return Path.Combine(PicturesPath, "Products"); } }

        public static ConfigInfo Config { get; set; }

        /// <summary>
        /// Used for client cache, to know when to reload new data
        /// </summary>
        public static int ProductsListVersion { get; set; } = -1;
    }

    public class ConfigInfo
    {
        public string CaptchaSecret { get; set; }

        public string CaptchaSiteKey { get; set; }
    }
}